import copy

import pytest
from icupy import (
    Locale, PluralRules, StringEnumeration, U_ICU_VERSION_MAJOR_NUM,
    UnicodeString,
)


def test_api():
    s1 = "a: n mod 10 is 2"
    s2 = "b: n mod 10 is 2"

    # static PluralRules *icu::PluralRules::createRules(
    #       const UnicodeString &description,
    #       UErrorCode &status
    # )
    rules1 = PluralRules.create_rules(UnicodeString(s1))
    assert isinstance(rules1, PluralRules)

    rules1a = PluralRules.create_rules(s1)
    assert isinstance(rules1a, PluralRules)

    rules1b = PluralRules.create_rules(s2)
    assert isinstance(rules1b, PluralRules)

    # [2]
    # icu::PluralRules::PluralRules(const PluralRules &other)
    rules2 = PluralRules(rules1)

    # UBool icu::PluralRules::operator!=(const PluralRules &other)
    assert not (rules1 != rules1a)
    assert rules1 != rules1b
    assert not (rules1 != rules2)

    # UBool icu::PluralRules::operator==(const PluralRules &other)
    assert rules1 == rules1a
    assert not (rules1 == rules1b)
    assert rules1 == rules2

    # UnicodeString icu::PluralRules::getKeywordOther()
    result = rules1.get_keyword_other()
    assert isinstance(result, UnicodeString)
    assert result == "other"

    # StringEnumeration *icu::PluralRules::getKeywords(UErrorCode &status)
    it = rules1.get_keywords()
    assert isinstance(it, StringEnumeration)
    assert list(it) == ["a", "other"]

    # UBool icu::PluralRules::isKeyword(const UnicodeString &keyword)
    assert rules1.is_keyword(UnicodeString("a"))
    assert rules1.is_keyword("a")
    assert not rules1.is_keyword(UnicodeString("b"))
    assert not rules1.is_keyword("b")


def test_clone():
    rules1 = PluralRules.create_default_rules()

    # PluralRules *icu::PluralRules::clone()
    rules2 = rules1.clone()
    assert isinstance(rules2, PluralRules)
    assert rules1 == rules2

    rules3 = copy.copy(rules1)
    assert rules1 == rules3

    rules4 = copy.deepcopy(rules1)
    assert rules1 == rules4


def test_create_default_rules():
    # static PluralRules *icu::PluralRules::createDefaultRules(
    #       UErrorCode &status
    # )
    rules = PluralRules.create_default_rules()
    assert isinstance(rules, PluralRules)


def test_for_locale():
    # [1]
    # static PluralRules *icu::PluralRules::forLocale(
    #       const Locale &locale,
    #       UErrorCode &status
    # )
    rules1 = PluralRules.for_locale(Locale("en"))
    assert isinstance(rules1, PluralRules)

    rules1a = PluralRules.for_locale("en")
    assert isinstance(rules1a, PluralRules)
    assert rules1 == rules1a


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 50, reason="ICU4C<50")
def test_for_locale_50():
    from icupy import UPluralType

    # [2]
    # static PluralRules *icu::PluralRules::forLocale(
    #       const Locale &locale,
    #       UPluralType type,
    #       UErrorCode &status
    # )
    rules2 = PluralRules.for_locale(
        Locale("en"),
        UPluralType.UPLURAL_TYPE_CARDINAL)
    assert isinstance(rules2, PluralRules)

    rules2a = PluralRules.for_locale(
        "en",
        UPluralType.UPLURAL_TYPE_CARDINAL)
    assert isinstance(rules2a, PluralRules)
    assert rules2 == rules2a


@pytest.mark.skip("FIXME: Enable test_get_samples().")
def test_get_samples():
    # FIXME: Enable test_get_samples().
    rules = PluralRules.for_locale(Locale("fr"))

    # [1]
    # int32_t icu::PluralRules::getSamples(
    #       const UnicodeString &keyword,
    #       double *dest,
    #       int32_t destCapacity,
    #       UErrorCode &status
    # )
    dest = rules.get_samples(UnicodeString("other"))
    assert isinstance(dest, list)
    assert len(dest) > 0
    assert all(isinstance(x, float) for x in dest)

    dest = rules.get_samples("other")
    assert isinstance(dest, list)
    assert len(dest) > 0
    assert all(isinstance(x, float) for x in dest)


def test_select():
    rules = PluralRules.create_rules("a: n not in 0..100;")

    # [5]
    # UnicodeString icu::PluralRules::select(double number)
    result = rules.select(-100.0)
    assert isinstance(result, UnicodeString)
    assert result == "other"

    assert rules.select(-100.1) == "a"

    # [6]
    # UnicodeString icu::PluralRules::select(int32_t number)
    result = rules.select(-100)
    assert isinstance(result, UnicodeString)
    assert result == "other"

    assert rules.select(-101) == "a"


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 64, reason="ICU4C<64")
def test_select_64():
    from icupy.number import NumberFormatter

    rules = PluralRules.create_rules("a: n not in 0..100;")
    fmt = NumberFormatter.with_locale("en")

    # [2]
    # UnicodeString icu::PluralRules::select(
    #       const number::FormattedNumber &number,
    #       UErrorCode &status
    # )
    result = rules.select(fmt.format_double(-100.0))
    assert isinstance(result, UnicodeString)
    assert result == "other"

    assert rules.select(fmt.format_double(-100.1)) == "a"


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 68, reason="ICU4C<68")
def test_select_68():
    from icupy import Formattable
    from icupy.number import NumberFormatter, NumberRangeFormatter

    rules = PluralRules.for_locale("fr")
    fmt = (NumberRangeFormatter.with_locale("fr")
           .number_formatter_first(NumberFormatter.with_())
           .number_formatter_second(NumberFormatter.with_()))

    # [3]
    # UnicodeString icu::PluralRules::select(
    #       const number::FormattedNumberRange &range,
    #       UErrorCode &status
    # )
    result = rules.select(
        fmt.format_formattable_range(Formattable(0), Formattable(2)))
    assert isinstance(result, UnicodeString)
    assert result == "other"

    assert rules.select(
        fmt.format_formattable_range(Formattable(0), Formattable(1))
    ) == "one"



import copy

import pytest
from icupy import (
    FieldPosition, FieldPositionIterator, Formattable, ICUError,
    ParsePosition, SelectFormat, UErrorCode, UnicodeString,
)


def test_api():
    # From icu/source/test/intltest/selfmts.cpp
    fmt = SelectFormat("feminine {feminineVerbValue} other{otherVerbValue}")

    # void icu::SelectFormat::applyPattern(
    #       const UnicodeString &pattern,
    #       UErrorCode &status
    # )
    pattern = UnicodeString(
        "masculine{masculineVerbValue} other{otherVerbValue}")
    fmt.apply_pattern(pattern)
    fmt.apply_pattern(str(pattern))

    # UnicodeString &icu::SelectFormat::toPattern(UnicodeString &appendTo)
    append_to = UnicodeString()
    result = fmt.to_pattern(append_to)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == pattern

    # UBool icu::SelectFormat::operator!=(const Format &other)
    fmt1 = SelectFormat("odd{foo} other{bar}")
    fmt2 = SelectFormat("odd{foo} other{bar}")
    fmt3 = SelectFormat("odd{foo} other{bar2}")
    assert not (fmt1 != fmt2)
    assert fmt1 != fmt3
    assert fmt2 != fmt3

    # UBool icu::SelectFormat::operator==(const Format &other)
    assert fmt1 == fmt2
    assert not (fmt1 == fmt3)
    assert not (fmt2 == fmt3)


def test_clone():
    fmt1 = SelectFormat("feminine {feminineVerbValue} other{otherVerbValue}")

    # SelectFormat *icu::SelectFormat::clone()
    fmt2 = fmt1.clone()
    assert isinstance(fmt2, SelectFormat)
    assert fmt1 == fmt2

    fmt3 = copy.copy(fmt1)
    assert fmt1 == fmt3

    fmt4 = copy.deepcopy(fmt1)
    assert fmt1 == fmt4


def test_format():
    fmt = SelectFormat("feminine {feminineVerbValue} other{otherVerbValue}")
    append_to = UnicodeString()

    # [1], [2]
    # UnicodeString &icu::SelectFormat::format(
    #       const Formattable &obj,
    #       UnicodeString &appendTo,
    #       FieldPosition &pos,
    #       UErrorCode &status
    # )
    append_to.remove()
    pos = FieldPosition(FieldPosition.DONT_CARE)
    result = fmt.format(
        Formattable(UnicodeString("feminine")),
        append_to,
        pos)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "feminineVerbValue"

    # *U_UNSUPPORTED_ERROR in ICU 69*
    # [3]
    # UnicodeString &icu::Format::format(
    #       const Formattable &obj,
    #       UnicodeString &appendTo,
    #       FieldPositionIterator *posIter,
    #       UErrorCode &status
    # )
    append_to.remove()
    pos_iter = FieldPositionIterator()
    with pytest.raises(ICUError) as exc_info:
        _ = fmt.format(
            Formattable(UnicodeString("feminine")),
            append_to,
            pos_iter)
    assert exc_info.value.args[0] == UErrorCode.U_UNSUPPORTED_ERROR

    append_to.remove()
    with pytest.raises(ICUError) as exc_info:
        _ = fmt.format(
            Formattable(UnicodeString("feminine")),
            append_to,
            None)
    assert exc_info.value.args[0] == UErrorCode.U_UNSUPPORTED_ERROR

    # [4]
    # UnicodeString &icu::Format::format(
    #       const Formattable &obj,
    #       UnicodeString &appendTo,
    #       UErrorCode &status
    # )
    append_to.remove()
    result = fmt.format(Formattable(UnicodeString("feminine")), append_to)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "feminineVerbValue"

    # [5]
    # UnicodeString &icu::SelectFormat::format(
    #       const UnicodeString &keyword,
    #       UnicodeString &appendTo,
    #       FieldPosition &pos,
    #       UErrorCode &status
    # )
    append_to.remove()
    pos = FieldPosition(FieldPosition.DONT_CARE)
    result = fmt.format(UnicodeString("feminine"), append_to, pos)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "feminineVerbValue"

    append_to.remove()
    pos = FieldPosition(FieldPosition.DONT_CARE)
    result = fmt.format("feminine", append_to, pos)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "feminineVerbValue"


def test_parse_object():
    fmt = SelectFormat("feminine {feminineVerbValue} other{otherVerbValue}")

    # void icu::SelectFormat::parseObject(
    #       const UnicodeString &source,
    #       Formattable &result,
    #       ParsePosition &parse_pos
    # )
    result = Formattable()
    parse_pos = ParsePosition()
    fmt.parse_object(UnicodeString("feminineVerbValue"), result, parse_pos)
    assert parse_pos.get_index() == 0
    assert parse_pos.get_error_index() != -1

    result = Formattable()
    parse_pos = ParsePosition()
    fmt.parse_object("feminineVerbValue", result, parse_pos)
    assert parse_pos.get_index() == 0
    assert parse_pos.get_error_index() != -1

    # *U_INVALID_FORMAT_ERROR in ICU 69*
    # void icu::Format::parseObject(
    #       const UnicodeString &source,
    #       Formattable &result,
    #       UErrorCode &status
    # )
    result = Formattable()
    with pytest.raises(ICUError) as exc_info:
        fmt.parse_object(UnicodeString("feminineVerbValue"), result)
    assert exc_info.value.args[0] == UErrorCode.U_INVALID_FORMAT_ERROR

    result = Formattable()
    with pytest.raises(ICUError) as exc_info:
        fmt.parse_object("feminineVerbValue", result)
    assert exc_info.value.args[0] == UErrorCode.U_INVALID_FORMAT_ERROR


def test_select_format():
    pattern = UnicodeString(
        "feminine {feminineVerbValue} other{otherVerbValue}")

    # [1]
    # icu::SelectFormat::SelectFormat(
    #       const UnicodeString &pattern,
    #       UErrorCode &status
    # )
    fmt1 = SelectFormat(pattern)

    fmt1a = SelectFormat(str(pattern))
    assert fmt1 == fmt1a

    # [2]
    # icu::SelectFormat::SelectFormat(const SelectFormat &other)
    fmt2 = SelectFormat(fmt1)
    assert fmt1 == fmt2



import pytest
from icupy import (
    DecimalFormatSymbols, Locale, NumberingSystem, StringEnumeration,
    UCurrencySpacing, ULocDataLocaleType, U_ICU_VERSION_MAJOR_NUM,
    UnicodeString,
)


def test_api():
    sym1 = DecimalFormatSymbols(Locale.get_us())
    sym2 = DecimalFormatSymbols(Locale.get_france())
    sym3 = DecimalFormatSymbols(sym1)

    # UBool icu::DecimalFormatSymbols::operator!=(
    #       const DecimalFormatSymbols &other
    # )
    assert sym1 != sym2
    assert not (sym1 != sym3)
    assert sym2 != sym3

    # UBool icu::DecimalFormatSymbols::operator==(
    #       const DecimalFormatSymbols &other
    # )
    assert not (sym1 == sym2)
    assert sym1 == sym3
    assert not (sym2 == sym3)

    # [1]
    # Locale icu::DecimalFormatSymbols::getLocale(void)
    result = sym1.get_locale()
    assert isinstance(result, Locale)
    assert result == Locale("en_US")

    # [2]
    # Locale icu::DecimalFormatSymbols::getLocale(
    #       ULocDataLocaleType type,
    #       UErrorCode &status
    # )
    result = sym1.get_locale(ULocDataLocaleType.ULOC_VALID_LOCALE)
    assert isinstance(result, Locale)
    assert result == Locale("en_US")

    # const UnicodeString &
    # icu::DecimalFormatSymbols::getPatternForCurrencySpacing(
    #       UCurrencySpacing type,
    #       UBool beforeCurrency,
    #       UErrorCode &status
    # )
    result = sym1.get_pattern_for_currency_spacing(
        UCurrencySpacing.UNUM_CURRENCY_INSERT,
        True)
    assert isinstance(result, UnicodeString)
    assert result == "\xa0"

    # UnicodeString icu::DecimalFormatSymbols::getSymbol(
    #       ENumberFormatSymbol symbol
    # )
    result = sym1.get_symbol(DecimalFormatSymbols.ZERO_DIGIT_SYMBOL)
    assert isinstance(result, UnicodeString)
    assert result == "0"

    # void icu::DecimalFormatSymbols::setPatternForCurrencySpacing(
    #       UCurrencySpacing type,
    #       UBool beforeCurrency,
    #       const UnicodeString &pattern
    # )
    sym1.set_pattern_for_currency_spacing(
        UCurrencySpacing.UNUM_CURRENCY_INSERT,
        True,
        UnicodeString())
    result = sym1.get_pattern_for_currency_spacing(
        UCurrencySpacing.UNUM_CURRENCY_INSERT,
        True)
    assert isinstance(result, UnicodeString)
    assert len(result) == 0

    sym1.set_pattern_for_currency_spacing(
        UCurrencySpacing.UNUM_CURRENCY_INSERT,
        True,
        "\xa0")
    result = sym1.get_pattern_for_currency_spacing(
        UCurrencySpacing.UNUM_CURRENCY_INSERT,
        True)
    assert isinstance(result, UnicodeString)
    assert result == "\xa0"

    # void icu::DecimalFormatSymbols::setSymbol(
    #       ENumberFormatSymbol symbol,
    #       const UnicodeString &value,
    #       const UBool propogateDigits = true
    # )
    sym1.set_symbol(
        DecimalFormatSymbols.ZERO_DIGIT_SYMBOL,
        UnicodeString("\uff10"),
        False)
    assert sym1.get_symbol(DecimalFormatSymbols.ZERO_DIGIT_SYMBOL) == "\uff10"
    assert sym1.get_symbol(DecimalFormatSymbols.ONE_DIGIT_SYMBOL) == "1"
    assert sym1.get_symbol(DecimalFormatSymbols.TWO_DIGIT_SYMBOL) == "2"
    assert sym1.get_symbol(DecimalFormatSymbols.THREE_DIGIT_SYMBOL) == "3"
    assert sym1.get_symbol(DecimalFormatSymbols.FOUR_DIGIT_SYMBOL) == "4"
    assert sym1.get_symbol(DecimalFormatSymbols.FIVE_DIGIT_SYMBOL) == "5"
    assert sym1.get_symbol(DecimalFormatSymbols.SIX_DIGIT_SYMBOL) == "6"
    assert sym1.get_symbol(DecimalFormatSymbols.SEVEN_DIGIT_SYMBOL) == "7"
    assert sym1.get_symbol(DecimalFormatSymbols.EIGHT_DIGIT_SYMBOL) == "8"
    assert sym1.get_symbol(DecimalFormatSymbols.NINE_DIGIT_SYMBOL) == "9"

    sym1.set_symbol(
        DecimalFormatSymbols.ZERO_DIGIT_SYMBOL,
        UnicodeString("\uff10"),
        True)
    assert sym1.get_symbol(DecimalFormatSymbols.ZERO_DIGIT_SYMBOL) == "\uff10"
    assert sym1.get_symbol(DecimalFormatSymbols.ONE_DIGIT_SYMBOL) == "\uff11"
    assert sym1.get_symbol(DecimalFormatSymbols.TWO_DIGIT_SYMBOL) == "\uff12"
    assert sym1.get_symbol(DecimalFormatSymbols.THREE_DIGIT_SYMBOL) == "\uff13"
    assert sym1.get_symbol(DecimalFormatSymbols.FOUR_DIGIT_SYMBOL) == "\uff14"
    assert sym1.get_symbol(DecimalFormatSymbols.FIVE_DIGIT_SYMBOL) == "\uff15"
    assert sym1.get_symbol(DecimalFormatSymbols.SIX_DIGIT_SYMBOL) == "\uff16"
    assert sym1.get_symbol(DecimalFormatSymbols.SEVEN_DIGIT_SYMBOL) == "\uff17"
    assert sym1.get_symbol(DecimalFormatSymbols.EIGHT_DIGIT_SYMBOL) == "\uff18"
    assert sym1.get_symbol(DecimalFormatSymbols.NINE_DIGIT_SYMBOL) == "\uff19"

    sym1.set_symbol(
        DecimalFormatSymbols.ZERO_DIGIT_SYMBOL,
        "0")
    assert sym1.get_symbol(DecimalFormatSymbols.ZERO_DIGIT_SYMBOL) == "0"
    assert sym1.get_symbol(DecimalFormatSymbols.ONE_DIGIT_SYMBOL) == "1"
    assert sym1.get_symbol(DecimalFormatSymbols.TWO_DIGIT_SYMBOL) == "2"
    assert sym1.get_symbol(DecimalFormatSymbols.THREE_DIGIT_SYMBOL) == "3"
    assert sym1.get_symbol(DecimalFormatSymbols.FOUR_DIGIT_SYMBOL) == "4"
    assert sym1.get_symbol(DecimalFormatSymbols.FIVE_DIGIT_SYMBOL) == "5"
    assert sym1.get_symbol(DecimalFormatSymbols.SIX_DIGIT_SYMBOL) == "6"
    assert sym1.get_symbol(DecimalFormatSymbols.SEVEN_DIGIT_SYMBOL) == "7"
    assert sym1.get_symbol(DecimalFormatSymbols.EIGHT_DIGIT_SYMBOL) == "8"
    assert sym1.get_symbol(DecimalFormatSymbols.NINE_DIGIT_SYMBOL) == "9"


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 52, reason="ICU4C<52")
def test_create_with_last_resort_data():
    # static DecimalFormatSymbols *
    # icu::DecimalFormatSymbols::createWithLastResortData(
    #       UErrorCode &status
    # )
    sym = DecimalFormatSymbols.create_with_last_resort_data()
    assert isinstance(sym, DecimalFormatSymbols)


def test_decimal_format_symbols():
    # [1]
    # icu::DecimalFormatSymbols::DecimalFormatSymbols(
    #       const Locale &locale,
    #       UErrorCode &status
    # )
    sym1 = DecimalFormatSymbols(Locale.get_us())
    sym1a = DecimalFormatSymbols("en_US")
    assert sym1 == sym1a

    # [2]
    # icu::DecimalFormatSymbols::DecimalFormatSymbols(
    #       const Locale &locale,
    #       const NumberingSystem &ns,
    #       UErrorCode &status
    # )
    sym2 = DecimalFormatSymbols(
        Locale.get_us(),
        NumberingSystem.create_instance())
    sym2a = DecimalFormatSymbols(
        "en_US",
        NumberingSystem.create_instance())
    assert sym2 == sym2a

    # [3]
    # icu::DecimalFormatSymbols::DecimalFormatSymbols(UErrorCode &status)
    sym3 = DecimalFormatSymbols()
    sym1b = DecimalFormatSymbols(Locale.get_default())
    assert sym3 == sym1b

    # [4]
    # icu::DecimalFormatSymbols::DecimalFormatSymbols(
    #       const DecimalFormatSymbols &
    # )
    sym4 = DecimalFormatSymbols(sym1)
    assert sym4 == sym1


def test_numbering_system():
    # [1]
    # static NumberingSystem *icu::NumberingSystem::createInstance(
    #       const Locale &inLocale,
    #       UErrorCode &status
    # )
    ns1 = NumberingSystem.create_instance(Locale.get_us())
    assert isinstance(ns1, NumberingSystem)

    ns2 = NumberingSystem.create_instance("en_US")
    assert isinstance(ns2, NumberingSystem)

    # [2]
    # static NumberingSystem *icu::NumberingSystem::createInstance(
    #       int32_t radix,
    #       UBool isAlgorithmic,
    #       const UnicodeString &description,
    #       UErrorCode &status
    # )
    ns3 = NumberingSystem.create_instance(
        10,
        False,
        UnicodeString("0123456789"))
    assert isinstance(ns3, NumberingSystem)

    ns4 = NumberingSystem.create_instance(
        10,
        False,
        "0123456789")
    assert isinstance(ns4, NumberingSystem)

    # [3]
    # static NumberingSystem *icu::NumberingSystem::createInstance(
    #       UErrorCode &status
    # )
    ns5 = NumberingSystem.create_instance()
    assert isinstance(ns5, NumberingSystem)

    # static NumberingSystem *icu::NumberingSystem::createInstanceByName(
    #       const char *name,
    #       UErrorCode &status
    # )
    ns6 = NumberingSystem.create_instance_by_name("latn")
    assert isinstance(ns6, NumberingSystem)

    # [2]
    # icu::NumberingSystem::NumberingSystem(const NumberingSystem &other)
    ns7 = NumberingSystem(ns1)

    # static StringEnumeration *icu::NumberingSystem::getAvailableNames(
    #       UErrorCode &status
    # )
    it = NumberingSystem.get_available_names()
    assert isinstance(it, StringEnumeration)
    assert len(it) > 0
    assert "latn" in it

    # UnicodeString icu::NumberingSystem::getDescription()
    result = ns7.get_description()
    assert isinstance(result, UnicodeString)
    assert result == "0123456789"

    # const char *icu::NumberingSystem::getName()
    result = ns7.get_name()
    assert isinstance(result, str)
    assert result == "latn"

    # int32_t icu::NumberingSystem::getRadix()
    assert ns7.get_radix() == 10

    # UBool icu::NumberingSystem::isAlgorithmic()
    assert not ns7.is_algorithmic()



import copy

import pytest
from icupy import (
    Calendar, GregorianCalendar, Locale, SimpleTimeZone, StringEnumeration,
    TimeZone, UCalendarDateFields, UCalendarDaysOfWeek, UCalendarMonths,
    UCalendarWallTimeOption, UCalendarWeekdayType, ULocDataLocaleType,
    U_ICU_VERSION_MAJOR_NUM, U_MILLIS_PER_HOUR as HOUR,
)


def test_api():
    # static const Locale *icu::Calendar::getAvailableLocales(
    #       int32_t &count
    # )
    result = Calendar.get_available_locales()
    assert isinstance(result, list)
    assert len(result) > 0
    assert all(isinstance(x, Locale) for x in result)

    # static StringEnumeration *icu::Calendar::getKeywordValuesForLocale(
    #       const char *key,
    #       const Locale &locale,
    #       UBool commonlyUsed,
    #       UErrorCode &status
    # )
    it1 = Calendar.get_keyword_values_for_locale(
        "calendar",
        Locale("he"),
        True)
    assert isinstance(it1, StringEnumeration)
    assert "hebrew" in it1

    it2 = Calendar.get_keyword_values_for_locale(
        "calendar",
        "he",
        True)
    assert isinstance(it2, StringEnumeration)
    assert "hebrew" in it2

    # static UDate icu::Calendar::getNow(void)
    result = Calendar.get_now()
    assert isinstance(result, float)
    assert result > 0

    when = 1215300615000.0  # 2008-07-05T23:30:15Z
    when2 = 1215304215000.0  # 2008-07-06T00:30:15Z
    when3 = 1215097200000.0  # 2008-07-04T00:00:00Z
    zone = TimeZone.get_gmt()
    zone2 = TimeZone.create_default()
    cal = Calendar.create_instance("en")
    assert isinstance(cal, GregorianCalendar)

    # const TimeZone &icu::Calendar::getTimeZone(void)
    result = cal.get_time_zone()
    assert isinstance(result, TimeZone)
    assert result == zone2

    # void icu::Calendar::setTimeZone(const TimeZone &zone)
    cal.set_time_zone(zone)
    assert cal.get_time_zone() == zone

    # void icu::Calendar::setTime(
    #       UDate date,
    #       UErrorCode &status
    # )
    cal.set_time(when)

    # UDate icu::Calendar::getTime(UErrorCode &status)
    assert cal.get_time() == when

    # void icu::Calendar::add(
    #       UCalendarDateFields field,
    #       int32_t amount,
    #       UErrorCode &status
    # )
    cal2 = cal.clone()
    cal2.add(UCalendarDateFields.UCAL_HOUR_OF_DAY, 1)
    assert cal2.get_time() == when2  # 2008-07-06T00:30:15Z

    # UBool icu::Calendar::after(
    #       const Calendar &when,
    #       UErrorCode &status
    # )
    assert not cal.after(cal2)
    assert cal2.after(cal)

    # UBool icu::Calendar::before(
    #       const Calendar &when,
    #       UErrorCode &status
    # )
    assert cal.before(cal2)
    assert not cal2.before(cal)

    # [2]
    # void icu::Calendar::clear(UCalendarDateFields field)
    cal3 = cal2.clone()
    cal3.clear(UCalendarDateFields.UCAL_SECOND)
    assert cal3.get_time() == 1215304200000.0  # 2008-07-06T00:30:00Z

    # [3]
    # void icu::Calendar::clear(void)
    cal3.clear()
    assert cal3.get_time() == 0

    cal4 = GregorianCalendar(zone, "en")
    cal4.set_time(when)

    # UBool icu::Calendar::equals(
    #       const Calendar &when,
    #       UErrorCode &status
    # )
    assert not cal.equals(cal2)
    assert not cal.equals(cal3)
    assert cal.equals(cal4)

    # int32_t icu::Calendar::fieldDifference(
    #       UDate when,
    #       UCalendarDateFields field,
    #       UErrorCode &status
    # )
    assert cal.field_difference(
        when2,
        UCalendarDateFields.UCAL_HOUR_OF_DAY) == 1
    assert cal.field_difference(
        when2,
        UCalendarDateFields.UCAL_MINUTE) == 0

    year = 2008
    month = UCalendarMonths.UCAL_JULY
    date = 5
    hour = 23
    minute = 30
    second = 15

    # [2]
    # void icu::Calendar::set(
    #       int32_t year,
    #       int32_t month,
    #       int32_t date
    # )
    cal3.set(year, month, date)

    # int32_t icu::Calendar::get(
    #       UCalendarDateFields field,
    #       UErrorCode &status
    # )
    assert cal3.get(UCalendarDateFields.UCAL_ERA) == GregorianCalendar.AD
    assert cal3.get(UCalendarDateFields.UCAL_YEAR) == year
    assert cal3.get(UCalendarDateFields.UCAL_MONTH) == month
    assert cal3.get(UCalendarDateFields.UCAL_DATE) == date
    assert cal3.get(UCalendarDateFields.UCAL_HOUR_OF_DAY) == 0
    assert cal3.get(UCalendarDateFields.UCAL_MINUTE) == 0
    assert cal3.get(UCalendarDateFields.UCAL_SECOND) == 0

    # [3]
    # void icu::Calendar::set(
    #       int32_t year,
    #       int32_t month,
    #       int32_t date,
    #       int32_t hour,
    #       int32_t minute
    # )
    cal3.set(year, month, date, hour, minute)
    assert cal3.get(UCalendarDateFields.UCAL_ERA) == GregorianCalendar.AD
    assert cal3.get(UCalendarDateFields.UCAL_YEAR) == year
    assert cal3.get(UCalendarDateFields.UCAL_MONTH) == month
    assert cal3.get(UCalendarDateFields.UCAL_DATE) == date
    assert cal3.get(UCalendarDateFields.UCAL_HOUR_OF_DAY) == hour
    assert cal3.get(UCalendarDateFields.UCAL_MINUTE) == minute
    assert cal3.get(UCalendarDateFields.UCAL_SECOND) == 0

    # [4]
    # void icu::Calendar::set(
    #       int32_t year,
    #       int32_t month,
    #       int32_t date,
    #       int32_t hour,
    #       int32_t minute,
    #       int32_t second
    # )
    cal3.set(year, month, date, hour - 1, minute, second)
    assert cal3.get(UCalendarDateFields.UCAL_ERA) == GregorianCalendar.AD
    assert cal3.get(UCalendarDateFields.UCAL_YEAR) == year
    assert cal3.get(UCalendarDateFields.UCAL_MONTH) == month
    assert cal3.get(UCalendarDateFields.UCAL_DATE) == date
    assert cal3.get(UCalendarDateFields.UCAL_HOUR_OF_DAY) == hour - 1
    assert cal3.get(UCalendarDateFields.UCAL_MINUTE) == minute
    assert cal3.get(UCalendarDateFields.UCAL_SECOND) == second

    # [5]
    # void icu::Calendar::set(
    #       UCalendarDateFields field,
    #       int32_t value
    # )
    cal3.set(UCalendarDateFields.UCAL_HOUR_OF_DAY, hour)
    assert cal3.get(UCalendarDateFields.UCAL_ERA) == GregorianCalendar.AD
    assert cal3.get(UCalendarDateFields.UCAL_YEAR) == year
    assert cal3.get(UCalendarDateFields.UCAL_MONTH) == month
    assert cal3.get(UCalendarDateFields.UCAL_DATE) == date
    assert cal3.get(UCalendarDateFields.UCAL_HOUR_OF_DAY) == hour
    assert cal3.get(UCalendarDateFields.UCAL_MINUTE) == minute
    assert cal3.get(UCalendarDateFields.UCAL_SECOND) == second

    # int32_t icu::Calendar::getActualMaximum(
    #       UCalendarDateFields field,
    #       UErrorCode &status
    # )
    assert cal.get_actual_maximum(UCalendarDateFields.UCAL_DAY_OF_MONTH) == 31

    # int32_t icu::Calendar::getLeastMaximum(UCalendarDateFields field)
    assert cal.get_least_maximum(UCalendarDateFields.UCAL_DAY_OF_MONTH) == 28

    # int32_t icu::Calendar::getMaximum(UCalendarDateFields field)
    assert cal.get_maximum(UCalendarDateFields.UCAL_DAY_OF_MONTH) == 31

    # int32_t icu::Calendar::getActualMinimum(
    #       UCalendarDateFields field,
    #       UErrorCode &status
    # )
    assert cal.get_actual_minimum(UCalendarDateFields.UCAL_DAY_OF_MONTH) == 1

    # int32_t icu::Calendar::getGreatestMinimum(UCalendarDateFields field)
    assert cal.get_greatest_minimum(UCalendarDateFields.UCAL_DAY_OF_MONTH) == 1

    # int32_t icu::Calendar::getMinimum(UCalendarDateFields field)
    assert cal.get_minimum(UCalendarDateFields.UCAL_DAY_OF_MONTH) == 1

    # UCalendarWeekdayType icu::Calendar::getDayOfWeekType(
    #       UCalendarDaysOfWeek dayOfWeek,
    #       UErrorCode &status
    # )
    assert (cal.get_day_of_week_type(UCalendarDaysOfWeek.UCAL_SUNDAY)
            == UCalendarWeekdayType.UCAL_WEEKEND)

    # UCalendarDaysOfWeek icu::Calendar::getFirstDayOfWeek(UErrorCode &status)
    assert cal2.get_first_day_of_week() == UCalendarDaysOfWeek.UCAL_SUNDAY

    # void icu::Calendar::setFirstDayOfWeek(UCalendarDaysOfWeek value)
    cal2.set_first_day_of_week(UCalendarDaysOfWeek.UCAL_MONDAY)
    assert cal2.get_first_day_of_week() == UCalendarDaysOfWeek.UCAL_MONDAY

    # Locale icu::Calendar::getLocale(
    #       ULocDataLocaleType type,
    #       UErrorCode &status
    # )
    result = cal.get_locale(ULocDataLocaleType.ULOC_ACTUAL_LOCALE)
    assert isinstance(result, Locale)
    assert result == Locale("en")

    # uint8_t icu::Calendar::getMinimalDaysInFirstWeek(void)
    assert cal2.get_minimal_days_in_first_week() == 1

    # void icu::Calendar::setMinimalDaysInFirstWeek(uint8_t value)
    cal2.set_minimal_days_in_first_week(7)
    assert cal2.get_minimal_days_in_first_week() == 7

    # int32_t icu::Calendar::getWeekendTransition(
    #       UCalendarDaysOfWeek dayOfWeek,
    #       UErrorCode &status
    # )
    assert (cal.get_weekend_transition(UCalendarDaysOfWeek.UCAL_SUNDAY)
            == 24 * HOUR)

    # UBool icu::Calendar::inDaylightTime(UErrorCode &status)
    assert not cal.in_daylight_time()
    assert not cal4.in_daylight_time()

    # UBool icu::Calendar::isEquivalentTo(const Calendar &other)
    assert not cal.is_equivalent_to(cal2)
    assert cal.is_equivalent_to(cal3)
    assert cal.is_equivalent_to(cal4)

    # UBool icu::Calendar::isLenient(void)
    assert cal2.is_lenient()

    # void icu::Calendar::setLenient(UBool lenient)
    cal2.set_lenient(False)
    assert not cal2.is_lenient()

    # UBool icu::Calendar::isSet(UCalendarDateFields field)
    assert cal2.is_set(UCalendarDateFields.UCAL_YEAR)
    assert cal2.is_set(UCalendarDateFields.UCAL_MONTH)
    assert cal2.is_set(UCalendarDateFields.UCAL_DATE)

    # [1]
    # UBool icu::Calendar::isWeekend(
    #       UDate date,
    #       UErrorCode &status
    # )
    assert cal2.is_weekend(when)
    assert not cal2.is_weekend(when3)

    # [2]
    # UBool icu::Calendar::isWeekend(void)
    assert cal2.is_weekend()

    # UBool icu::Calendar::operator!=(const Calendar &that)
    assert cal != cal2
    assert cal != cal3
    assert cal != cal4

    # UBool icu::Calendar::operator==(const Calendar &that)
    assert not (cal == cal2)
    assert not (cal == cal3)
    assert not (cal == cal4)

    # TimeZone *icu::Calendar::orphanTimeZone(void)
    zone4 = cal2.orphan_time_zone()
    assert isinstance(zone4, TimeZone)
    assert zone4 != zone2
    assert cal2.get_time_zone() == zone2

    # [3]
    # void icu::Calendar::roll(
    #       UCalendarDateFields field,
    #       int32_t amount,
    #       UErrorCode &status
    # )
    cal3.roll(UCalendarDateFields.UCAL_HOUR_OF_DAY, 1)
    # 2008-07-05T23:30:15Z -> 2008-07-05T00:30:15Z
    assert cal3.get_time() == 1215217815000.0  # 2008-07-05T00:30:15Z

    # UDate icu::GregorianCalendar::getGregorianChange(void)
    assert cal4.get_gregorian_change() == -12219292800000.0  # 1582-10-15T00:00:00Z

    # void icu::GregorianCalendar::setGregorianChange(
    #       UDate date,
    #       UErrorCode &success
    # )
    cal4.set_gregorian_change(-2208988800000.0)  # 1900-01-01T00:00:00Z
    assert cal4.get_gregorian_change() == -2208988800000.0

    # UBool icu::GregorianCalendar::isLeapYear(int32_t year)
    assert cal4.is_leap_year(2008)
    assert not cal4.is_leap_year(2009)


def test_calendar_create_instance():
    locale1 = Locale.get_english()
    locale2 = Locale.get_default()
    zone1 = SimpleTimeZone(8 * HOUR, "s2")
    zone2 = TimeZone.create_default()

    # [1]
    # static Calendar *icu::Calendar::createInstance(
    #       const Locale &aLocale,
    #       UErrorCode &success
    # )
    cal1 = Calendar.create_instance(locale1)
    assert isinstance(cal1, GregorianCalendar)
    assert cal1.get_locale(ULocDataLocaleType.ULOC_ACTUAL_LOCALE) == locale1
    assert cal1.get_time_zone() == zone2

    assert Calendar.create_instance("en").get_locale(
        ULocDataLocaleType.ULOC_ACTUAL_LOCALE) == locale1

    # [2]
    # static Calendar *icu::Calendar::createInstance(
    #       const TimeZone &zone,
    #       const Locale &aLocale,
    #       UErrorCode &success
    # )
    cal2 = Calendar.create_instance(zone1, locale1)
    assert isinstance(cal2, GregorianCalendar)
    assert cal2.get_locale(ULocDataLocaleType.ULOC_ACTUAL_LOCALE) == locale1
    assert cal2.get_time_zone() == zone1

    assert Calendar.create_instance(zone1, "en").get_locale(
        ULocDataLocaleType.ULOC_ACTUAL_LOCALE) == locale1

    # [3]
    # static Calendar *icu::Calendar::createInstance(
    #       const TimeZone &zone,
    #       UErrorCode &success
    # )
    cal3 = Calendar.create_instance(zone1)
    assert isinstance(cal3, GregorianCalendar)
    assert cal3.get_locale(ULocDataLocaleType.ULOC_VALID_LOCALE) == locale2
    assert cal3.get_time_zone() == zone1

    # [6]
    # static Calendar *icu::Calendar::createInstance(UErrorCode &success)
    cal6 = Calendar.create_instance()
    assert isinstance(cal6, GregorianCalendar)
    assert cal6.get_locale(ULocDataLocaleType.ULOC_VALID_LOCALE) == locale2
    assert cal6.get_time_zone() == zone2


def test_clone():
    cal1 = GregorianCalendar(TimeZone.get_gmt())
    cal1.set_time(1215298800000.0)  # 2008-07-05T23:00:00Z

    # Calendar *icu::Calendar::clone()
    cal2 = cal1.clone()
    assert isinstance(cal2, GregorianCalendar)

    cal3 = copy.copy(cal1)

    cal4 = copy.deepcopy(cal1)
    assert cal1 == cal2 == cal3 == cal4


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 49, reason="ICU4C<49")
def test_get_repeated_wall_time_option():
    zone = SimpleTimeZone(8 * HOUR, "s2")
    cal = Calendar.create_instance(zone)

    # UCalendarWallTimeOption icu::Calendar::getRepeatedWallTimeOption(void)
    assert (cal.get_repeated_wall_time_option()
            == UCalendarWallTimeOption.UCAL_WALLTIME_LAST)

    # void icu::Calendar::setRepeatedWallTimeOption(
    #       UCalendarWallTimeOption option
    # )
    cal.set_repeated_wall_time_option(
        UCalendarWallTimeOption.UCAL_WALLTIME_FIRST)
    assert (cal.get_repeated_wall_time_option()
            == UCalendarWallTimeOption.UCAL_WALLTIME_FIRST)


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 49, reason="ICU4C<49")
def test_get_skipped_wall_time_option():
    zone = SimpleTimeZone(8 * HOUR, "s2")
    cal = Calendar.create_instance(zone)

    # UCalendarWallTimeOption icu::Calendar::getSkippedWallTimeOption(void)
    assert (cal.get_skipped_wall_time_option()
            == UCalendarWallTimeOption.UCAL_WALLTIME_LAST)

    # void icu::Calendar::setSkippedWallTimeOption(
    #       UCalendarWallTimeOption option
    # )
    cal.set_skipped_wall_time_option(
        UCalendarWallTimeOption.UCAL_WALLTIME_FIRST)
    assert (cal.get_skipped_wall_time_option()
            == UCalendarWallTimeOption.UCAL_WALLTIME_FIRST)


def test_get_time_zone_upcasting():
    from icupy import BasicTimeZone

    cal = Calendar.create_instance()

    cal.set_time_zone(TimeZone.get_gmt())
    zone = cal.get_time_zone()
    assert isinstance(zone, SimpleTimeZone)

    zone = cal.orphan_time_zone()
    assert isinstance(zone, SimpleTimeZone)

    # TimeZone -> BasicTimeZone
    cal.set_time_zone(TimeZone.create_time_zone("JST"))
    zone = cal.get_time_zone()
    assert not isinstance(zone, SimpleTimeZone)
    assert isinstance(zone, BasicTimeZone)

    zone = cal.orphan_time_zone()
    assert not isinstance(zone, SimpleTimeZone)
    assert isinstance(zone, BasicTimeZone)


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 49, reason="ICU4C<49")
def test_get_type():
    # const char *icu::Calendar::getType()
    cal1 = Calendar.create_instance("ja")
    result = cal1.get_type()
    assert isinstance(result, str)
    assert result == "gregorian"

    cal2 = Calendar.create_instance("ja@calendar=japanese")
    result = cal2.get_type()
    assert isinstance(result, str)
    assert result == "japanese"


def test_gregorian_calendar():
    assert issubclass(GregorianCalendar, Calendar)
    locale1 = Locale.get_english()
    locale2 = Locale.get_default()
    zone1 = SimpleTimeZone(8 * HOUR, "s2")
    zone2 = TimeZone.create_default()

    # [1]
    # icu::GregorianCalendar::GregorianCalendar(UErrorCode &success)
    cal1 = GregorianCalendar()
    assert cal1.get_locale(ULocDataLocaleType.ULOC_VALID_LOCALE) == locale2
    assert cal1.get_time_zone() == zone2

    # [3]
    # icu::GregorianCalendar::GregorianCalendar(
    #       const TimeZone &zone,
    #       UErrorCode &success
    # )
    cal3 = GregorianCalendar(zone1)
    assert cal3.get_locale(ULocDataLocaleType.ULOC_VALID_LOCALE) == locale2
    assert cal3.get_time_zone() == zone1

    # [4]
    # icu::GregorianCalendar::GregorianCalendar(
    #       const Locale &aLocale,
    #       UErrorCode &success
    # )
    cal4 = GregorianCalendar(locale1)
    assert cal4.get_locale(ULocDataLocaleType.ULOC_ACTUAL_LOCALE) == locale1
    assert cal4.get_time_zone() == zone2

    # [6]
    # icu::GregorianCalendar::GregorianCalendar(
    #       const TimeZone &zone,
    #       const Locale &aLocale,
    #       UErrorCode &success
    # )
    cal6 = GregorianCalendar(zone1, locale1)
    assert cal6.get_locale(ULocDataLocaleType.ULOC_ACTUAL_LOCALE) == locale1
    assert cal6.get_time_zone() == zone1

    # [7]
    # icu::GregorianCalendar::GregorianCalendar(
    #       int32_t year,
    #       int32_t month,
    #       int32_t date,
    #       UErrorCode &success
    # )
    cal7 = GregorianCalendar(2008, UCalendarMonths.UCAL_JULY, 5)
    assert cal7.get_locale(ULocDataLocaleType.ULOC_VALID_LOCALE) == locale2
    assert cal7.get_time_zone() == zone2
    assert cal7.get_time() == 1215183600000.0  # 2008-07-05T00:00:00Z

    # [8]
    # icu::GregorianCalendar::GregorianCalendar(
    #       int32_t year,
    #       int32_t month,
    #       int32_t date,
    #       int32_t hour,
    #       int32_t minute,
    #       UErrorCode &success
    # )
    cal8 = GregorianCalendar(2008, UCalendarMonths.UCAL_JULY, 5, 23, 30)
    assert cal8.get_locale(ULocDataLocaleType.ULOC_VALID_LOCALE) == locale2
    assert cal8.get_time_zone() == zone2
    assert cal8.get_time() == 1215268200000.0  # 2008-07-05T23:30:00Z

    # [9]
    # icu::GregorianCalendar::GregorianCalendar(
    #       int32_t year,
    #       int32_t month,
    #       int32_t date,
    #       int32_t hour,
    #       int32_t minute,
    #       int32_t second,
    #       UErrorCode &success
    # )
    cal9 = GregorianCalendar(2008, UCalendarMonths.UCAL_JULY, 5, 23, 30, 15)
    assert cal9.get_locale(ULocDataLocaleType.ULOC_VALID_LOCALE) == locale2
    assert cal9.get_time_zone() == zone2
    assert cal9.get_time() == 1215268215000.0  # 2008-07-05T23:30:15Z

    # [10]
    # icu::GregorianCalendar::GregorianCalendar(
    #       const GregorianCalendar &source
    # )
    cal10 = GregorianCalendar(cal9)
    assert cal10.get_locale(ULocDataLocaleType.ULOC_VALID_LOCALE) == locale2
    assert cal10.get_time_zone() == zone2
    assert cal10.get_time() == 1215268215000.0  # 2008-07-05T23:30:15Z



import pytest
from icupy import U_ICU_VERSION_MAJOR_NUM

if U_ICU_VERSION_MAJOR_NUM < 65:
    pytest.skip("ICU4C<65", allow_module_level=True)

from icupy import (
    ErrorCode, Locale, LocaleMatcher, UErrorCode, ULocMatchDemotion,
    ULocMatchFavorSubtag,
)


def test_builder():
    locales = [Locale("fr"), Locale("en_GB"), Locale("en")]

    # icu::LocaleMatcher::Builder::Builder()
    #
    # Builder &icu::LocaleMatcher::Builder::addSupportedLocale(
    #       const Locale &locale
    # )
    #
    # LocaleMatcher icu::LocaleMatcher::Builder::build(UErrorCode &errorCode)
    matcher = (LocaleMatcher.Builder()
               .add_supported_locale(locales[0])
               .add_supported_locale("en_GB")
               .add_supported_locale(locales[2])
               .build())
    assert isinstance(matcher, LocaleMatcher)

    # [1]
    # const Locale *icu::LocaleMatcher::getBestMatch(
    #       const Locale &desiredLocale,
    #       UErrorCode &errorCode
    # )
    assert matcher.get_best_match(Locale("en_US")) == Locale("en")
    assert matcher.get_best_match("en_US") == Locale("en")

    # [2]
    # Builder &icu::LocaleMatcher::Builder::setSupportedLocales(
    #       Locale::Iterator &locales
    # )
    #
    # Builder &icu::LocaleMatcher::Builder::setDefaultLocale(
    #       const Locale *defaultLocale
    # )
    matcher = (LocaleMatcher.Builder()
               .set_supported_locales(locales)
               .set_default_locale(Locale.get_german())
               .build())
    assert matcher.get_best_match(Locale("ja_JP")) == Locale("de")
    assert matcher.get_best_match("ja_JP") == Locale("de")

    # Builder &icu::LocaleMatcher::Builder::setDemotionPerDesiredLocale(
    #       ULocMatchDemotion demotion
    # )
    supported = [Locale("fr"), Locale("de-CH"), Locale("it")]
    desired = [Locale("fr-CH"), Locale("de-CH"), Locale("it")]
    matcher = (LocaleMatcher.Builder()
               .set_supported_locales(supported)
               .set_demotion_per_desired_locale(
        ULocMatchDemotion.ULOCMATCH_DEMOTION_NONE)
               .build())

    # [2]
    # const Locale *icu::LocaleMatcher::getBestMatch(
    #       Locale::Iterator &desiredLocales,
    #       UErrorCode &errorCode
    # )
    assert matcher.get_best_match(desired) == Locale("de_CH")

    # Builder &icu::LocaleMatcher::Builder::setFavorSubtag(
    #       ULocMatchFavorSubtag subtag
    # )
    matcher = (LocaleMatcher.Builder()
               .set_supported_locales(locales)
               .set_favor_subtag(ULocMatchFavorSubtag.ULOCMATCH_FAVOR_SCRIPT)
               .build())
    assert matcher.get_best_match(Locale("ja")) == Locale("fr")
    assert matcher.get_best_match("ja") == Locale("fr")

    # Builder &icu::LocaleMatcher::Builder::setSupportedLocalesFromListString(
    #       StringPiece locales
    # )
    matcher = (LocaleMatcher.Builder()
               .set_supported_locales_from_list_string(
        " el, fr;q=0.555555, en-GB ; q = 0.88  , el; q =0, en;q=0.88 , fr ")
               .build())
    assert matcher.get_best_match_for_list_string(
        "el, fr, fr;q=0, en-GB") == Locale("en_GB")
    assert matcher.get_best_match(Locale("en_US")) == Locale("en")

    # UBool icu::LocaleMatcher::Builder::copyErrorTo(UErrorCode &outErrorCode)
    bld = LocaleMatcher.Builder().set_supported_locales(locales)
    bld.build()
    out_error_code = ErrorCode()
    assert not bld.copy_error_to(out_error_code)
    assert out_error_code.get() == UErrorCode.U_ZERO_ERROR


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 67, reason="ICU4C<67")
def test_builder_set_direction():
    from icupy import ULocMatchDirection

    # Builder &icu::LocaleMatcher::Builder::setDirection(
    #       ULocMatchDirection direction
    # )
    supported = [Locale("ar"), Locale("nn")]
    desired = [Locale("arz-EG"), Locale("nb-DK")]
    matcher = (LocaleMatcher.Builder()
               .set_supported_locales(supported)
               .set_direction(
        ULocMatchDirection.ULOCMATCH_DIRECTION_ONLY_TWO_WAY)
               .build())
    assert matcher.get_best_match(desired) == Locale("nn")


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 68, reason="ICU4C<68")
def test_builder_set_max_distance():
    # Builder &icu::LocaleMatcher::Builder::setMaxDistance(
    #       const Locale &desired,
    #       const Locale &supported
    # )
    matcher = (LocaleMatcher.Builder()
               .set_max_distance(Locale("de-AT"), Locale.get_german())
               .build())
    assert matcher.is_match(Locale("de-LU"), Locale.get_german())
    assert not matcher.is_match(Locale("da"), Locale("no"))
    assert not matcher.is_match(Locale.get_chinese(),
                                Locale.get_traditional_chinese())

    matcher = (LocaleMatcher.Builder()
               .set_max_distance("de-AT", Locale("de"))
               .build())
    assert not matcher.is_match("da", Locale("no"))

    matcher = (LocaleMatcher.Builder()
               .set_max_distance(Locale("de-AT"), "de")
               .build())
    assert matcher.is_match(Locale("de-LU"), "de")

    matcher = (LocaleMatcher.Builder()
               .set_max_distance("de-AT", "de")
               .build())
    assert not matcher.is_match("zh", "zh_TW")


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 68, reason="ICU4C<68")
def test_builder_set_no_default_locale():
    # Builder &icu::LocaleMatcher::Builder::setNoDefaultLocale()
    locales = [Locale("fr"), Locale("en_GB"), Locale("en")]
    matcher = (LocaleMatcher.Builder()
               .set_supported_locales(locales)
               .set_no_default_locale()
               .build())
    assert matcher.get_best_match(Locale("en_GB")) == Locale("en_GB")
    assert matcher.get_best_match("en_GB") == Locale("en_GB")
    assert matcher.get_best_match(Locale("ja_JP")) is None
    assert matcher.get_best_match("ja_JP") is None


def test_get_best_match_result():
    locales = [Locale("fr"), Locale("en-GB")]
    matcher = (LocaleMatcher.Builder()
               .set_supported_locales(locales)
               .add_supported_locale(Locale.get_english())
               .set_default_locale(Locale.get_german())
               .build())

    # [1]
    # Result icu::LocaleMatcher::getBestMatchResult(
    #       const Locale &desiredLocale,
    #       UErrorCode &errorCode
    # )
    desired_locale = Locale("en_GB")
    result = matcher.get_best_match_result(desired_locale)
    assert result.get_desired_index() == 0
    assert result.get_desired_locale() == desired_locale
    assert result.get_supported_index() == 1
    assert result.get_supported_locale() == Locale("en_GB")
    resolved = result.make_resolved_locale()
    assert isinstance(resolved, Locale)
    assert resolved == desired_locale

    desired_locale = Locale("ja_JP")
    result = matcher.get_best_match_result(desired_locale)
    assert result.get_desired_index() == -1
    assert result.get_desired_locale() is None
    assert result.get_supported_index() == -1
    assert result.get_supported_locale() == Locale("de")
    assert result.make_resolved_locale() == Locale("de")

    # [2]
    # Result icu::LocaleMatcher::getBestMatchResult(
    #       Locale::Iterator &desiredLocales,
    #       UErrorCode &errorCode
    # )
    desired_locales = [Locale("ja-JP"), Locale("en-US")]
    result = matcher.get_best_match_result(desired_locales)
    assert result.get_desired_index() == 1
    assert result.get_desired_locale() == desired_locales[1]
    assert result.get_supported_index() == 2
    assert result.get_supported_locale() == Locale("en")
    assert result.make_resolved_locale() == Locale("en_US")



import copy
from pathlib import Path

import pytest
from icupy import (
    ICUError, Locale, ResourceBundle, UErrorCode, ULocDataLocaleType,
    UnicodeString, UResType,
    ures_close, ures_open, ures_open_direct,
)


def _int28_to_uint28(n: int) -> int:
    return n if n >= 0 else 2 ** 28 + n


def test_api():
    # UResourceBundle *ures_openDirect(
    #       const char *packageName,
    #       const char *locale,
    #       UErrorCode *status
    # )
    rb = ures_open_direct(None, "metadata")
    test1 = ResourceBundle(rb)
    ures_close(rb)

    # UResType icu::ResourceBundle::getType(void)
    assert test1.get_type() == UResType.URES_TABLE

    # int32_t icu::ResourceBundle::getSize(void)
    assert test1.get_size() > 0
    assert len(test1) == test1.get_size()

    # const char *icu::ResourceBundle::getKey(void)
    assert test1.get_key() is None

    # const char *icu::ResourceBundle::getName(void)
    assert test1.get_name() == "metadata"

    # [1]
    # ResourceBundle icu::ResourceBundle::get(
    #       const char *key,
    #       UErrorCode &status
    # )
    test2 = test1.get("alias")
    assert isinstance(test2, ResourceBundle)
    assert test2.get_type() == UResType.URES_TABLE
    assert test2.get_size() > 0
    assert len(test2) == test2.get_size()
    assert test2.get_key() == "alias"
    assert test2.get_name() == "metadata"

    test3 = test2.get("language")
    assert isinstance(test3, ResourceBundle)
    assert test3.get_type() == UResType.URES_TABLE
    assert test3.get_size() > 0
    assert len(test3) == test3.get_size()
    assert test3.get_key() == "language"
    assert test3.get_name() == "metadata"

    test4 = test3.get("sh")
    assert isinstance(test4, ResourceBundle)
    assert test4.get_type() == UResType.URES_TABLE
    assert test4.get_size() >= 2
    assert len(test4) == test4.get_size()
    assert test4.get_key() == "sh"
    assert test4.get_name() == "metadata"

    # [1]
    # UnicodeString icu::ResourceBundle::getStringEx(
    #       const char *key,
    #       UErrorCode &status
    # )
    result = test4.get_string_ex("replacement")
    assert isinstance(result, UnicodeString)
    assert result == "sr_Latn"

    result = test4.get_string_ex("reason")
    assert isinstance(result, UnicodeString)
    assert result == "legacy"

    # [2]
    # UnicodeString icu::ResourceBundle::getStringEx(
    #       int32_t index,
    #       UErrorCode &status
    # )
    result1 = test4.get_string_ex(0)
    assert isinstance(result1, UnicodeString)
    result2 = test4.get_string_ex(1)
    assert isinstance(result2, UnicodeString)
    assert result1 != result2
    assert str(result1) in ["sr_Latn", "legacy"]
    assert str(result2) in ["sr_Latn", "legacy"]


def test_api2():
    path = Path(__file__).resolve().parent / "testdata"
    rb = None
    try:
        rb = ures_open_direct(str(path), "testtypes")
    except ICUError as ex:
        if ex.args[0] != UErrorCode.U_MISSING_RESOURCE_ERROR:
            raise
        pytest.skip("testdata.dat is not found (not an error). "
                    "You need to build a test data from the source. "
                    "See also <icu4c>/icu/source/test/testdata/")

    test1 = ResourceBundle(rb)
    ures_close(rb)

    # "testtypes"
    assert test1.get_type() == UResType.URES_TABLE
    assert test1.get_size() > 0
    assert test1.get_key() is None
    assert test1.get_name() == "testtypes"

    resources = list()

    # UBool icu::ResourceBundle::hasNext(void)
    while test1.has_next():
        # ResourceBundle icu::ResourceBundle::getNext(UErrorCode &status)
        result = test1.get_next()
        assert isinstance(result, ResourceBundle)
        resources.append(result)

    assert not test1.has_next()
    with pytest.raises(ICUError) as exc_info:
        _ = test1.get_next()
    assert exc_info.value.args[0] == UErrorCode.U_INDEX_OUTOFBOUNDS_ERROR

    # void icu::ResourceBundle::resetIterator(void)
    test1.reset_iterator()
    assert test1.has_next()

    for index, expected in enumerate(resources):
        # [2]
        # ResourceBundle icu::ResourceBundle::get(
        #       int32_t index,
        #       UErrorCode &status
        # )
        result = test1.get(index)
        assert isinstance(result, ResourceBundle)
        assert result.get_key() is not None
        assert result.get_key() == expected.get_key()

    resources2 = [x for x in test1]
    assert len(resources2) == len(resources)
    for res1, res2 in zip(resources, resources2):
        assert res1.get_key() is not None
        assert res1.get_key() == res2.get_key()

    # "testtypes" > "binarytest"
    test2 = test1.get("binarytest")
    assert isinstance(test2, ResourceBundle)
    assert test2.get_type() == UResType.URES_BINARY

    # const uint8_t *icu::ResourceBundle::getBinary(
    #       int32_t &len,
    #       UErrorCode &status
    # )
    result = test2.get_binary()
    assert isinstance(result, list)
    assert len(result) == 15
    assert result == [
        0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a,
        0x0b, 0x0c, 0x0d, 0x0e
    ]

    # "testtypes" > "emptybin"
    test3 = test1.get("emptybin")
    assert isinstance(test3, ResourceBundle)
    assert test3.get_type() == UResType.URES_BINARY
    result = test3.get_binary()
    assert isinstance(result, list)
    assert len(result) == 0

    # "testtypes" > "minusone"
    test4 = test1.get("minusone")
    assert isinstance(test4, ResourceBundle)
    assert test4.get_type() == UResType.URES_INT

    # int32_t icu::ResourceBundle::getInt(UErrorCode &status)
    result = test4.get_int()
    assert isinstance(result, int)
    assert result == -1

    # uint32_t icu::ResourceBundle::getUInt(UErrorCode &status)
    result = test4.get_uint()
    assert isinstance(result, int)
    assert result == _int28_to_uint28(-1)

    # "testtypes" > "emptyint"
    test5 = test1.get("emptyint")
    assert isinstance(test5, ResourceBundle)
    assert test5.get_type() == UResType.URES_INT
    result = test5.get_int()
    assert isinstance(result, int)
    assert result == 0

    result = test5.get_uint()
    assert isinstance(result, int)
    assert result == 0

    # "testtypes" > "integerarray"
    test6 = test1.get("integerarray")
    assert isinstance(test6, ResourceBundle)
    assert test6.get_type() == UResType.URES_INT_VECTOR

    # const int32_t *icu::ResourceBundle::getIntVector(
    #       int32_t &len,
    #       UErrorCode &status
    # )
    result = test6.get_int_vector()
    assert isinstance(result, list)
    assert result == [1, 2, 3, -3, 4, 5, 6, 7]

    # "testtypes" > "emptyintv"
    test7 = test1.get("emptyintv")
    assert isinstance(test7, ResourceBundle)
    assert test7.get_type() == UResType.URES_INT_VECTOR
    result = test7.get_int_vector()
    assert isinstance(result, list)
    assert len(result) == 0

    # "testtypes" > "menu" > "file"
    test8 = test1.get("menu")
    assert isinstance(test8, ResourceBundle)
    test9 = test8.get("file")
    assert isinstance(test9, ResourceBundle)

    # [2]
    # UnicodeString icu::ResourceBundle::getNextString(
    #       const char **key,
    #       UErrorCode &status
    # )
    result = test9.get_next_string()
    assert isinstance(result, UnicodeString)
    assert str(result) in ["Open", "Save", "Exit"]

    result = test9.get_next_string()
    assert isinstance(result, UnicodeString)
    assert str(result) in ["Open", "Save", "Exit"]

    result = test9.get_next_string()
    assert isinstance(result, UnicodeString)
    assert str(result) in ["Open", "Save", "Exit"]

    with pytest.raises(ICUError) as exc_info:
        _ = test9.get_next_string()
    assert exc_info.value.args[0] == UErrorCode.U_INDEX_OUTOFBOUNDS_ERROR

    # "testtypes" > "zerotest"
    test10 = test1.get("zerotest")
    assert isinstance(test10, ResourceBundle)
    assert test10.get_type() == UResType.URES_STRING

    # UnicodeString icu::ResourceBundle::getString(UErrorCode &status)
    result = test10.get_string()
    assert isinstance(result, UnicodeString)
    assert len(result) == 7
    assert str(result) == "abc\u0000def"

    # "testtypes" > "emptystring"
    test11 = test1.get("emptystring")
    assert isinstance(test11, ResourceBundle)
    assert test11.get_type() == UResType.URES_STRING
    result = test11.get_string()
    assert isinstance(result, UnicodeString)
    assert len(result) == 0

    # "default"
    test12 = ResourceBundle(str(path), Locale.get_us())

    # void icu::ResourceBundle::getVersion(UVersionInfo versionInfo)
    version_info = test12.get_version()
    assert isinstance(version_info, list)
    assert len(version_info) == 4
    assert version_info == [44, 0, 0, 0]

    test13 = ResourceBundle(str(path), Locale("te_IN"))
    version_info = test13.get_version()
    assert isinstance(version_info, list)
    assert len(version_info) == 4
    assert version_info == [55, 0, 0, 0]


def test_clone():
    test1 = ResourceBundle(None, Locale.get_us())

    # ResourceBundle *icu::ResourceBundle::clone()
    test2 = test1.clone()
    assert isinstance(test2, ResourceBundle)
    assert test1.get_name() == test2.get_name()
    assert test1.get_key() == test2.get_key()

    test3 = copy.copy(test1)
    assert test3.get_name() == test1.get_name()
    assert test3.get_key() == test1.get_key()

    test4 = copy.deepcopy(test1)
    assert test4.get_name() == test1.get_name()
    assert test4.get_key() == test1.get_key()


def test_resource_bundle():
    default_locale = Locale.get_default()
    if str(default_locale) == "ja_JP":
        res_locale = Locale.get_us()
    else:
        res_locale = Locale.get_japan()
    assert default_locale != res_locale

    # [1]
    # icu::ResourceBundle::ResourceBundle(
    #       const UnicodeString &packageName,
    #       const Locale &locale,
    #       UErrorCode &err
    # )
    test1 = ResourceBundle(UnicodeString(), res_locale)
    assert test1.get_type() == UResType.URES_TABLE
    assert test1.get_size() >= 0
    assert test1.get_key() is None
    assert (test1.get_locale(ULocDataLocaleType.ULOC_ACTUAL_LOCALE)
            == res_locale)
    assert (test1.get_locale(ULocDataLocaleType.ULOC_VALID_LOCALE)
            == res_locale)
    assert test1.get_name() == res_locale.get_name()

    test1b = ResourceBundle(UnicodeString(), str(res_locale))
    assert test1b.get_type() == UResType.URES_TABLE
    assert test1b.get_size() >= 0
    assert test1b.get_key() is None
    assert (test1b.get_locale(ULocDataLocaleType.ULOC_ACTUAL_LOCALE)
            == res_locale)
    assert (test1b.get_locale(ULocDataLocaleType.ULOC_VALID_LOCALE)
            == res_locale)
    assert test1b.get_name() == res_locale.get_name()

    # [2]
    # icu::ResourceBundle::ResourceBundle(
    #       const UnicodeString &packageName,
    #       UErrorCode &err
    # )
    test2 = ResourceBundle(UnicodeString())
    assert test2.get_type() == UResType.URES_TABLE
    assert test2.get_size() >= 0
    assert test2.get_key() is None
    assert (test2.get_locale(ULocDataLocaleType.ULOC_ACTUAL_LOCALE)
            == default_locale)
    assert (test2.get_locale(ULocDataLocaleType.ULOC_VALID_LOCALE)
            == default_locale)
    assert test2.get_name() == default_locale.get_name()

    test2a = ResourceBundle("")
    assert test2a.get_type() == UResType.URES_TABLE
    assert test2a.get_size() >= 0
    assert test2a.get_key() is None
    assert (test2a.get_locale(ULocDataLocaleType.ULOC_ACTUAL_LOCALE)
            == default_locale)
    assert (test2a.get_locale(ULocDataLocaleType.ULOC_VALID_LOCALE)
            == default_locale)
    assert test2a.get_name() == default_locale.get_name()

    # [3]
    # icu::ResourceBundle::ResourceBundle(UErrorCode &err)
    test3 = ResourceBundle()
    assert test3.get_type() == UResType.URES_TABLE
    assert test3.get_size() >= 0
    assert test3.get_key() is None
    assert (test3.get_locale(ULocDataLocaleType.ULOC_ACTUAL_LOCALE)
            == default_locale)
    assert (test3.get_locale(ULocDataLocaleType.ULOC_VALID_LOCALE)
            == default_locale)
    assert test3.get_name() == default_locale.get_name()

    # [4]
    # icu::ResourceBundle::ResourceBundle(
    #       const char *packageName,
    #       const Locale &locale,
    #       UErrorCode &err
    # )
    test4 = ResourceBundle(None, res_locale)
    assert test4.get_type() == UResType.URES_TABLE
    assert test4.get_size() >= 0
    assert test4.get_key() is None
    assert (test4.get_locale(ULocDataLocaleType.ULOC_ACTUAL_LOCALE)
            == res_locale)
    assert (test4.get_locale(ULocDataLocaleType.ULOC_VALID_LOCALE)
            == res_locale)
    assert test4.get_name() == res_locale.get_name()

    test4a = ResourceBundle(None, str(res_locale))
    assert test4a.get_type() == UResType.URES_TABLE
    assert test4a.get_size() >= 0
    assert test4a.get_key() is None
    assert (test4a.get_locale(ULocDataLocaleType.ULOC_ACTUAL_LOCALE)
            == res_locale)
    assert (test4a.get_locale(ULocDataLocaleType.ULOC_VALID_LOCALE)
            == res_locale)
    assert test4a.get_name() == res_locale.get_name()

    # [5]
    # icu::ResourceBundle::ResourceBundle(const ResourceBundle &original)
    test5 = ResourceBundle(test1)
    assert test5.get_type() == UResType.URES_TABLE
    assert test5.get_size() >= 0
    assert test5.get_key() is None
    assert (test5.get_locale(ULocDataLocaleType.ULOC_ACTUAL_LOCALE)
            == res_locale)
    assert (test5.get_locale(ULocDataLocaleType.ULOC_VALID_LOCALE)
            == res_locale)
    assert test5.get_name() == res_locale.get_name()

    # [6]
    # icu::ResourceBundle::ResourceBundle(
    #       UResourceBundle *res,
    #       UErrorCode &status
    # )
    rb = ures_open(None, str(res_locale))
    test6 = ResourceBundle(rb)
    ures_close(rb)
    assert test6.get_type() == UResType.URES_TABLE
    assert test6.get_size() >= 0
    assert test6.get_key() is None
    assert (test6.get_locale(ULocDataLocaleType.ULOC_ACTUAL_LOCALE)
            == res_locale)
    assert (test6.get_locale(ULocDataLocaleType.ULOC_VALID_LOCALE)
            == res_locale)
    assert test6.get_name() == res_locale.get_name()



import pytest
from icupy import U_ICU_VERSION_MAJOR_NUM

if U_ICU_VERSION_MAJOR_NUM < 64:
    pytest.skip("ICU4C<64", allow_module_level=True)

from icupy import ErrorCode, ICUError, Locale, LocaleBuilder, UErrorCode


def test_api():
    # icu::LocaleBuilder::LocaleBuilder()
    bld = LocaleBuilder()

    # LocaleBuilder &icu::LocaleBuilder::addUnicodeLocaleAttribute(
    #       StringPiece attribute
    # )
    result = bld.add_unicode_locale_attribute("abc")
    assert isinstance(result, LocaleBuilder)
    assert id(result) == id(bld)

    # LocaleBuilder &icu::LocaleBuilder::removeUnicodeLocaleAttribute(
    #       StringPiece attribute
    # )
    result = bld.remove_unicode_locale_attribute("abc")
    assert isinstance(result, LocaleBuilder)
    assert id(result) == id(bld)

    # LocaleBuilder &icu::LocaleBuilder::setExtension(
    #       char key,
    #       StringPiece value
    # )
    result = bld.set_extension("U", "co-pinyin")
    assert isinstance(result, LocaleBuilder)
    assert id(result) == id(bld)

    # LocaleBuilder &icu::LocaleBuilder::setLanguage(StringPiece language)
    result = bld.set_language("zh")
    assert isinstance(result, LocaleBuilder)
    assert id(result) == id(bld)

    # LocaleBuilder &icu::LocaleBuilder::setRegion(StringPiece region)
    result = bld.set_region("HK")
    assert isinstance(result, LocaleBuilder)
    assert id(result) == id(bld)

    # LocaleBuilder &icu::LocaleBuilder::setScript(StringPiece script)
    result = bld.set_script("Hans")
    assert isinstance(result, LocaleBuilder)
    assert id(result) == id(bld)

    # LocaleBuilder &icu::LocaleBuilder::setUnicodeLocaleKeyword(
    #       StringPiece key,
    #       StringPiece type
    # )
    result = bld.set_unicode_locale_keyword("nu", "latn")
    assert isinstance(result, LocaleBuilder)
    assert id(result) == id(bld)

    # LocaleBuilder &icu::LocaleBuilder::setVariant(StringPiece variant)
    result = bld.set_variant("revised")
    assert isinstance(result, LocaleBuilder)
    assert id(result) == id(bld)

    # Locale icu::LocaleBuilder::build(UErrorCode &status)
    loc = (bld
           .set_language("zh")
           .set_script("Hant")
           .set_region("SG")
           .set_region("HK")
           .set_script("Hans")
           .set_variant("revised")
           .set_unicode_locale_keyword("nu", "thai")
           .set_extension("U", "co-pinyin")
           .set_unicode_locale_keyword("nu", "latn")
           .build())
    assert isinstance(loc, Locale)
    assert loc.to_language_tag() == "zh-Hans-HK-revised-u-co-pinyin-nu-latn"

    loc = (bld
           .set_unicode_locale_keyword("nu", None)
           .set_unicode_locale_keyword("co", None)
           .set_script("")
           .set_variant("")
           .set_region("")
           .build())
    assert loc.to_language_tag() == "zh"

    bld = LocaleBuilder()
    loc = (bld
           .set_language("fr")
           .add_unicode_locale_attribute("abc")
           .add_unicode_locale_attribute("aBc")
           .add_unicode_locale_attribute("EFG")
           .add_unicode_locale_attribute("efghi")
           .add_unicode_locale_attribute("efgh")
           .add_unicode_locale_attribute("efGhi")
           .add_unicode_locale_attribute("EFg")
           .add_unicode_locale_attribute("hijk")
           .add_unicode_locale_attribute("EFG")
           .add_unicode_locale_attribute("HiJK")
           .add_unicode_locale_attribute("aBc")
           .build())
    assert loc.to_language_tag() == "fr-u-abc-efg-efgh-efghi-hijk"

    # LocaleBuilder &icu::LocaleBuilder::setLanguageTag(StringPiece tag)
    loc = bld.set_language_tag("de-DE-u-co-phonebk").build()
    assert loc.to_language_tag() == "de-DE-u-co-phonebk"

    # LocaleBuilder &icu::LocaleBuilder::setLocale(const Locale &locale)
    loc2 = LocaleBuilder().set_locale(loc).build()
    assert loc2 == loc

    loc2 = LocaleBuilder().set_locale("de_DE@collation=phonebook").build()
    assert loc2 == loc

    # LocaleBuilder &icu::LocaleBuilder::clearExtensions()
    loc = bld.clear_extensions().build()
    assert loc.to_language_tag() == "de-DE"

    # LocaleBuilder &icu::LocaleBuilder::clear()
    loc = bld.clear().build()
    assert loc == Locale("")


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 65, reason="ICU4C<65")
def test_copy_error_to():
    bld = LocaleBuilder()
    bld.set_language("en").build()

    # UBool icu::LocaleBuilder::copyErrorTo(UErrorCode &outErrorCode)
    out_error_code = ErrorCode()
    assert not bld.copy_error_to(out_error_code)
    assert out_error_code.get() == UErrorCode.U_ZERO_ERROR

    with pytest.raises(ICUError) as exc_info:
        bld.set_unicode_locale_keyword("123", "abc").build()
    assert exc_info.value.args[0] == UErrorCode.U_ILLEGAL_ARGUMENT_ERROR
    out_error_code = ErrorCode()
    assert bld.copy_error_to(out_error_code)
    assert out_error_code.get() == UErrorCode.U_ILLEGAL_ARGUMENT_ERROR



from icupy import (
    Collator, CollationElementIterator, Locale, StringCharacterIterator,
    UColAttribute, UColAttributeValue, UnicodeString,
)


def test_api():
    coll = Collator.create_instance(Locale("es"))

    # CollationElementIterator *
    # icu::RuleBasedCollator::createCollationElementIterator(
    #       const UnicodeString &source
    # )
    source = UnicodeString("cha")
    it1 = coll.create_collation_element_iterator(source)
    assert isinstance(it1, CollationElementIterator)
    assert it1.get_offset() == 0

    # int32_t icu::CollationElementIterator::next(UErrorCode &status)
    order1 = it1.next()
    assert order1 != CollationElementIterator.NULLORDER
    order2 = it1.next()
    assert order2 != CollationElementIterator.NULLORDER
    order3 = it1.next()
    assert order3 != CollationElementIterator.NULLORDER
    assert it1.next() == CollationElementIterator.NULLORDER

    # int32_t icu::CollationElementIterator::getOffset(void)
    assert it1.get_offset() != 0
    it1.reset()
    assert it1.get_offset() == 0

    # void icu::CollationElementIterator::setOffset(
    #       int32_t newOffset,
    #       UErrorCode &status
    # )
    it1.set_offset(1)
    assert list(it1) == [order1, order2, order3]

    # int32_t icu::CollationElementIterator::getMaxExpansion(int32_t order)
    assert it1.get_max_expansion(order1) == 1
    assert it1.get_max_expansion(order2) == 1
    assert it1.get_max_expansion(order3) == 1

    # UBool icu::CollationElementIterator::isIgnorable(int32_t order)
    assert not CollationElementIterator.is_ignorable(order1)
    assert not CollationElementIterator.is_ignorable(order2)
    assert not CollationElementIterator.is_ignorable(order3)

    # int32_t icu::CollationElementIterator::previous(UErrorCode &status)
    it1.set_offset(3)
    assert it1.get_offset() == 3
    assert it1.previous() == order3
    assert it1.previous() == order2
    assert it1.previous() == order1
    assert it1.previous() == CollationElementIterator.NULLORDER

    # icu::CollationElementIterator::CollationElementIterator(
    #       const CollationElementIterator &other
    # )
    it2 = CollationElementIterator(it1)
    assert it1 == it2
    assert not (it1 != it2)

    # [1]
    # void icu::CollationElementIterator::setText(
    #       CharacterIterator &str,
    #       UErrorCode &status
    # )
    source = StringCharacterIterator(UnicodeString("ca"))
    it2.set_text(source)
    it1.reset()
    assert not (it1 == it2)
    assert it1 != it2

    source = StringCharacterIterator(UnicodeString("cha"))
    it2.set_text(source)
    assert it1 == it2
    assert not (it1 != it2)

    # [2]
    # void icu::CollationElementIterator::setText(
    #       const UnicodeString &str,
    #       UErrorCode &status
    # )
    source = UnicodeString("cha")
    it2.set_text(source)
    assert it1 == it2
    assert not (it1 != it2)

    it2.set_text("cha")
    assert it1 == it2
    assert not (it1 != it2)

    # int32_t icu::CollationElementIterator::primaryOrder(int32_t order)
    assert (CollationElementIterator.primary_order(order1)
            == CollationElementIterator.primary_order(it2.next()))
    assert (CollationElementIterator.primary_order(order2)
            == CollationElementIterator.primary_order(it2.next()))
    assert (CollationElementIterator.primary_order(order3)
            == CollationElementIterator.primary_order(it2.next()))

    # int32_t icu::CollationElementIterator::secondaryOrder(int32_t order)
    it2.reset()
    assert (CollationElementIterator.secondary_order(order1)
            == CollationElementIterator.secondary_order(it2.next()))
    assert (CollationElementIterator.secondary_order(order2)
            == CollationElementIterator.secondary_order(it2.next()))
    assert (CollationElementIterator.secondary_order(order3)
            == CollationElementIterator.secondary_order(it2.next()))

    # int32_t icu::CollationElementIterator::tertiaryOrder(int32_t order)
    it2.reset()
    assert (CollationElementIterator.tertiary_order(order1)
            == CollationElementIterator.tertiary_order(it2.next()))
    assert (CollationElementIterator.tertiary_order(order2)
            == CollationElementIterator.tertiary_order(it2.next()))
    assert (CollationElementIterator.tertiary_order(order3)
            == CollationElementIterator.tertiary_order(it2.next()))

    # int32_t icu::CollationElementIterator::strengthOrder(int32_t order)
    coll.set_attribute(UColAttribute.UCOL_STRENGTH,
                       UColAttributeValue.UCOL_PRIMARY)
    assert it1.strength_order(order1) == order1 & 0xffff0000
    assert it1.strength_order(order2) == order2 & 0xffff0000
    assert it1.strength_order(order3) == order3 & 0xffff0000

    coll.set_attribute(UColAttribute.UCOL_STRENGTH,
                       UColAttributeValue.UCOL_SECONDARY)
    assert it1.strength_order(order1) == order1 & 0xffffff00
    assert it1.strength_order(order2) == order2 & 0xffffff00
    assert it1.strength_order(order3) == order3 & 0xffffff00

    coll.set_attribute(UColAttribute.UCOL_STRENGTH,
                       UColAttributeValue.UCOL_TERTIARY)
    assert it1.strength_order(order1) == order1
    assert it1.strength_order(order2) == order2
    assert it1.strength_order(order3) == order3



import copy

import pytest
from icupy import (
    ICUError, Locale, StringEnumeration, UErrorCode,
    U_ICU_VERSION_MAJOR_NUM, UnicodeString,
)


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 63, reason="ICU4C<63")
def test_add_likely_subtags():
    # void icu::Locale::addLikelySubtags(UErrorCode &status)
    loc = Locale("sr")
    loc.add_likely_subtags()
    assert loc.get_name() == "sr_Cyrl_RS"


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 67, reason="ICU4C<67")
def test_canonicalize():
    # "aa_Deva_IN_SAAHO@calendar=japanese"
    loc = Locale.for_language_tag("aa-Deva-IN-SAAHO-u-ca-japanese")

    # void icu::Locale::canonicalize(UErrorCode &status)
    loc.canonicalize()

    # "ssy_Deva_IN@calendar=japanese"
    assert loc.to_language_tag() == "ssy-Deva-IN-u-ca-japanese"


def test_clone():
    test1 = Locale.get_us()

    # Locale *icu::Locale::clone()
    test2 = test1.clone()
    assert isinstance(test2, Locale)
    assert id(test1) != id(test2)
    assert test1.get_name() == test2.get_name()

    test3 = copy.copy(test1)
    assert test1.get_name() == test3.get_name()

    test4 = copy.deepcopy(test1)
    assert test1.get_name() == test4.get_name()


def test_constants():
    # static const Locale &icu::Locale::getCanada(void)
    loc = Locale.get_canada()
    assert isinstance(loc, Locale)
    assert loc.get_name() == "en_CA"
    assert Locale.CANADA == "en_CA"

    # static const Locale &icu::Locale::getCanadaFrench(void)
    loc = Locale.get_canada_french()
    assert isinstance(loc, Locale)
    assert loc.get_name() == "fr_CA"
    assert Locale.CANADA_FRENCH == "fr_CA"

    # static const Locale &icu::Locale::getChina(void)
    loc = Locale.get_china()
    assert isinstance(loc, Locale)
    assert loc.get_name() == "zh_CN"
    assert Locale.CHINA == "zh_CN"

    # static const Locale &icu::Locale::getChinese(void)
    loc = Locale.get_chinese()
    assert isinstance(loc, Locale)
    assert loc.get_name() == "zh"
    assert Locale.CHINESE == "zh"

    # static const Locale &icu::Locale::getEnglish(void)
    loc = Locale.get_english()
    assert isinstance(loc, Locale)
    assert loc.get_name() == "en"
    assert Locale.ENGLISH == "en"

    # static const Locale &icu::Locale::getFrance(void)
    loc = Locale.get_france()
    assert isinstance(loc, Locale)
    assert loc.get_name() == "fr_FR"
    assert Locale.FRANCE == "fr_FR"

    # static const Locale &icu::Locale::getFrench(void)
    loc = Locale.get_french()
    assert isinstance(loc, Locale)
    assert loc.get_name() == "fr"
    assert Locale.FRENCH == "fr"

    # static const Locale &icu::Locale::getGerman(void)
    loc = Locale.get_german()
    assert isinstance(loc, Locale)
    assert loc.get_name() == "de"
    assert Locale.GERMAN == "de"

    # static const Locale &icu::Locale::getGermany(void)
    loc = Locale.get_germany()
    assert isinstance(loc, Locale)
    assert loc.get_name() == "de_DE"
    assert Locale.GERMANY == "de_DE"

    # static const Locale &icu::Locale::getItalian(void)
    loc = Locale.get_italian()
    assert isinstance(loc, Locale)
    assert loc.get_name() == "it"
    assert Locale.ITALIAN == "it"

    # static const Locale &icu::Locale::getItaly(void)
    loc = Locale.get_italy()
    assert isinstance(loc, Locale)
    assert loc.get_name() == "it_IT"
    assert Locale.ITALY == "it_IT"

    # static const Locale &icu::Locale::getJapan(void)
    loc = Locale.get_japan()
    assert isinstance(loc, Locale)
    assert loc.get_name() == "ja_JP"
    assert Locale.JAPAN == "ja_JP"

    # static const Locale &icu::Locale::getJapanese(void)
    loc = Locale.get_japanese()
    assert isinstance(loc, Locale)
    assert loc.get_name() == "ja"
    assert Locale.JAPANESE == "ja"

    # static const Locale &icu::Locale::getKorea(void)
    loc = Locale.get_korea()
    assert isinstance(loc, Locale)
    assert loc.get_name() == "ko_KR"
    assert Locale.KOREA == "ko_KR"

    # static const Locale &icu::Locale::getKorean(void)
    loc = Locale.get_korean()
    assert isinstance(loc, Locale)
    assert loc.get_name() == "ko"
    assert Locale.KOREAN == "ko"

    # static const Locale &icu::Locale::getPRC(void)
    loc = Locale.get_prc()
    assert isinstance(loc, Locale)
    assert loc.get_name() == "zh_CN"
    assert Locale.PRC == "zh_CN"

    # static const Locale &icu::Locale::getRoot(void)
    loc = Locale.get_root()
    assert isinstance(loc, Locale)
    assert len(loc.get_name()) == 0

    # static const Locale &icu::Locale::getSimplifiedChinese(void)
    loc = Locale.get_simplified_chinese()
    assert isinstance(loc, Locale)
    assert loc.get_name() == "zh_CN"
    assert Locale.SIMPLIFIED_CHINESE == "zh_CN"

    # static const Locale &icu::Locale::getTaiwan(void)
    loc = Locale.get_taiwan()
    assert isinstance(loc, Locale)
    assert loc.get_name() == "zh_TW"
    assert Locale.TAIWAN == "zh_TW"

    # static const Locale &icu::Locale::getTraditionalChinese(void)
    loc = Locale.get_traditional_chinese()
    assert isinstance(loc, Locale)
    assert loc.get_name() == "zh_TW"
    assert Locale.TRADITIONAL_CHINESE == "zh_TW"

    # static const Locale &icu::Locale::getUK(void)
    loc = Locale.get_uk()
    assert isinstance(loc, Locale)
    assert loc.get_name() == "en_GB"
    assert Locale.UK == "en_GB"

    # static const Locale &icu::Locale::getUS(void)
    loc = Locale.get_us()
    assert isinstance(loc, Locale)
    assert loc.get_name() == "en_US"
    assert Locale.US == "en_US"


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 67, reason="ICU4C<67")
def test_create_canonical():
    # static Locale icu::Locale::createCanonical(const char *name)
    loc = Locale.create_canonical("aa-Deva-IN-SAAHO-u-ca-japanese")
    assert isinstance(loc, Locale)
    assert loc.to_language_tag() == "ssy-Deva-IN-u-ca-japanese"


def test_create_from_name():
    # static Locale icu::Locale::createFromName(const char *name)
    loc = Locale.create_from_name(None)
    assert isinstance(loc, Locale)
    assert loc == Locale.get_default()

    loc = Locale.create_from_name("x-klingon-latn-zx.utf32be@special")
    assert isinstance(loc, Locale)
    assert loc.get_language() == "x-klingon"
    assert loc.get_script() == "Latn"
    assert loc.get_country() == "ZX"
    assert len(loc.get_variant()) == 0


def test_create_keywords():
    # StringEnumeration *icu::Locale::createKeywords(UErrorCode &status)
    loc = Locale("de@calendar=buddhist;collation=phonebook")
    result = loc.create_keywords()
    assert isinstance(result, StringEnumeration)
    assert len(result) == 2
    t = [x for x in result]
    assert "calendar" in t
    assert "collation" in t

    loc = Locale("de")
    result = loc.create_keywords()
    assert result is None

    loc.set_to_bogus()
    result = loc.create_keywords()
    assert result is None


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 63, reason="ICU4C<63")
def test_create_unicode_keywords():
    # StringEnumeration *icu::Locale::createUnicodeKeywords(UErrorCode &status)
    loc = Locale("de@calendar=buddhist;collation=phonebook")
    result = loc.create_unicode_keywords()
    assert isinstance(result, StringEnumeration)
    assert len(result) == 2
    t = [x for x in result]
    assert "ca" in t
    assert "co" in t

    loc = Locale("de")
    result = loc.create_unicode_keywords()
    assert result is None

    loc.set_to_bogus()
    result = loc.create_unicode_keywords()
    assert result is None


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 63, reason="ICU4C<63")
def test_for_language_tag():
    # static Locale icu::Locale::forLanguageTag(
    #       StringPiece tag,
    #       UErrorCode &status
    # )
    loc = Locale.for_language_tag("af-t-ar-i0-handwrit-u-ca-coptic-x-foo")
    assert isinstance(loc, Locale)
    assert loc.get_name() == "af@calendar=coptic;t=ar-i0-handwrit;x=foo"


def test_get_available_locales():
    # static const Locale *icu::Locale::getAvailableLocales(int32_t &count)
    result = Locale.get_available_locales()
    assert isinstance(result, list)
    assert len(result) > 0
    assert all(isinstance(x, Locale) for x in result)
    assert Locale.get_default() in result
    assert Locale.get_root() not in result


def test_get_base_name():
    # const char *icu::Locale::getBaseName()
    loc = Locale("de_DE@calendar=buddhist;collation=phonebook")
    assert loc.get_base_name() == "de_DE"


def test_get_country():
    # const char *icu::Locale::getCountry()
    loc = Locale("de_DE@calendar=buddhist;collation=phonebook")
    assert loc.get_country() == "DE"


def test_get_default():
    # static const Locale &icu::Locale::getDefault(void)
    loc = Locale.get_default()
    assert isinstance(loc, Locale)
    assert loc == Locale.create_from_name(None)
    assert len(loc.get_name()) > 0


def test_get_display_country():
    default_locale = None
    try:
        default_locale = Locale.get_default()
        loc = Locale.get_us()
        Locale.set_default(loc)
        display_locale = Locale.get_france()
        disp_country = UnicodeString()

        # [1]
        # UnicodeString &icu::Locale::getDisplayCountry(
        #       const Locale &displayLocale,
        #       UnicodeString &dispCountry
        # )
        result = loc.get_display_country(display_locale, disp_country)
        assert isinstance(result, UnicodeString)
        assert id(result) == id(disp_country)
        assert disp_country == UnicodeString("\\u00C9tats-Unis").unescape()

        disp_country.remove()
        result = loc.get_display_country("fr", disp_country)
        assert isinstance(result, UnicodeString)
        assert id(result) == id(disp_country)
        assert disp_country == UnicodeString("\\u00C9tats-Unis").unescape()

        # [2]
        # UnicodeString &icu::Locale::getDisplayCountry(
        #       UnicodeString &dispCountry
        # )
        disp_country.remove()
        result = loc.get_display_country(disp_country)
        assert isinstance(result, UnicodeString)
        assert id(result) == id(disp_country)
        assert disp_country == "United States"
    finally:
        if default_locale:
            Locale.set_default(default_locale)


def test_get_display_language():
    default_locale = None
    try:
        default_locale = Locale.get_default()
        loc = Locale.get_us()
        Locale.set_default(loc)
        display_locale = Locale.get_france()
        disp_lang = UnicodeString()

        # [1]
        # UnicodeString &icu::Locale::getDisplayLanguage(
        #       const Locale &displayLocale,
        #       UnicodeString &dispLang
        # )
        result = loc.get_display_language(display_locale, disp_lang)
        assert isinstance(result, UnicodeString)
        assert id(result) == id(disp_lang)
        assert disp_lang == "anglais"

        disp_lang.remove()
        result = loc.get_display_language("fr", disp_lang)
        assert isinstance(result, UnicodeString)
        assert id(result) == id(disp_lang)
        assert disp_lang == "anglais"

        # [2]
        # UnicodeString &icu::Locale::getDisplayLanguage(
        #       UnicodeString &dispLang
        # )
        disp_lang.remove()
        result = loc.get_display_language(disp_lang)
        assert isinstance(result, UnicodeString)
        assert id(result) == id(disp_lang)
        assert disp_lang == "English"
    finally:
        if default_locale:
            Locale.set_default(default_locale)


def test_get_display_name():
    default_locale = None
    try:
        default_locale = Locale.get_default()
        loc = Locale.get_us()
        Locale.set_default(loc)
        display_locale = Locale.get_france()
        name = UnicodeString()

        # [1]
        # UnicodeString &icu::Locale::getDisplayName(
        #       const Locale &displayLocale,
        #       UnicodeString &name
        # )
        result = loc.get_display_name(display_locale, name)
        assert isinstance(result, UnicodeString)
        assert id(result) == id(name)
        assert name == UnicodeString("anglais (\\u00C9tats-Unis)").unescape()

        name.remove()
        result = loc.get_display_name("fr", name)
        assert isinstance(result, UnicodeString)
        assert id(result) == id(name)
        assert name == UnicodeString("anglais (\\u00C9tats-Unis)").unescape()

        # [2]
        # UnicodeString &icu::Locale::getDisplayName(UnicodeString &name)
        name.remove()
        result = loc.get_display_name(name)
        assert isinstance(result, UnicodeString)
        assert id(result) == id(name)
        assert name == "English (United States)"
    finally:
        if default_locale:
            Locale.set_default(default_locale)


def test_get_display_script():
    default_locale = None
    try:
        default_locale = Locale.get_default()
        loc = Locale.get_us()
        Locale.set_default(loc)
        loc = Locale("zh", "Hans", "CN")
        display_locale = Locale.get_france()
        disp_script = UnicodeString()

        # [1]
        # UnicodeString &icu::Locale::getDisplayScript(
        #       const Locale &displayLocale,
        #       UnicodeString &dispScript
        # )
        result = loc.get_display_script(display_locale, disp_script)
        assert isinstance(result, UnicodeString)
        assert id(result) == id(disp_script)
        assert (disp_script
                == UnicodeString("sinogrammes simplifi\\u00E9s").unescape())

        disp_script.remove()
        result = loc.get_display_script("fr", disp_script)
        assert isinstance(result, UnicodeString)
        assert id(result) == id(disp_script)
        assert (disp_script
                == UnicodeString("sinogrammes simplifi\\u00E9s").unescape())

        # [2]
        # UnicodeString &icu::Locale::getDisplayScript(
        #       UnicodeString &dispScript
        # )
        disp_script.remove()
        result = loc.get_display_script(disp_script)
        assert isinstance(result, UnicodeString)
        assert id(result) == id(disp_script)
        assert disp_script == "Simplified Han"
    finally:
        if default_locale:
            Locale.set_default(default_locale)


def test_get_display_variant():
    default_locale = None
    try:
        default_locale = Locale.get_default()
        loc = Locale.get_us()
        Locale.set_default(loc)
        loc = Locale("no", "NO", "NY")
        display_locale = Locale.get_france()
        disp_var = UnicodeString()

        # [1]
        # UnicodeString &icu::Locale::getDisplayVariant(
        #       const Locale &displayLocale,
        #       UnicodeString &dispVar
        # )
        result = loc.get_display_variant(display_locale, disp_var)
        assert isinstance(result, UnicodeString)
        assert id(result) == id(disp_var)
        assert disp_var == "NY"

        disp_var.remove()
        result = loc.get_display_variant("fr", disp_var)
        assert isinstance(result, UnicodeString)
        assert id(result) == id(disp_var)
        assert disp_var == "NY"

        # [2]
        # UnicodeString &icu::Locale::getDisplayVariant(UnicodeString &dispVar)
        disp_var.remove()
        result = loc.get_display_variant(disp_var)
        assert isinstance(result, UnicodeString)
        assert id(result) == id(disp_var)
        assert disp_var == "NY"
    finally:
        if default_locale:
            Locale.set_default(default_locale)


def test_get_iso3_country():
    # const char *icu::Locale::getISO3Country()
    loc = Locale.get_us()
    assert loc.get_iso3_country() == "USA"


def test_get_iso3_language():
    # const char *icu::Locale::getISO3Language()
    loc = Locale.get_us()
    assert loc.get_iso3_language() == "eng"


def test_get_iso_countries():
    # static const char * const *icu::Locale::getISOCountries()
    result = Locale.get_iso_countries()
    assert isinstance(result, list)
    assert len(result) > 0
    assert all(isinstance(x, str) for x in result)
    assert all(len(x) == 2 for x in result)


def test_get_iso_languages():
    # static const char * const *icu::Locale::getISOLanguages()
    result = Locale.get_iso_languages()
    assert isinstance(result, list)
    assert len(result) > 0
    assert all(isinstance(x, str) for x in result)
    assert all(2 <= len(x) <= 3 for x in result)


def test_get_keyword_value():
    loc = Locale("de@calendar=buddhist;collation=phonebook")

    # [3]
    # template<typename StringClass>
    # StringClass icu::Locale::getKeywordValue(
    #       StringPiece keywordName,
    #       UErrorCode &status
    # )
    value = loc.get_keyword_value("calendar")
    assert isinstance(value, str)
    assert value == "buddhist"

    value = loc.get_keyword_value("collation")
    assert isinstance(value, str)
    assert value == "phonebook"

    value = loc.get_keyword_value("currency")
    assert isinstance(value, str)
    assert len(value) == 0

    loc.set_to_bogus()
    with pytest.raises(ICUError) as exc_info:
        _ = loc.get_keyword_value("calendar")
    assert exc_info.value.args[0] == UErrorCode.U_ILLEGAL_ARGUMENT_ERROR


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 63, reason="ICU4C<63")
def test_get_keywords():
    loc = Locale("de@calendar=buddhist;collation=phonebook")

    # void icu::Locale::getKeywords(
    #       OutputIterator iterator,
    #       UErrorCode &status
    # )
    result = loc.get_keywords()
    assert isinstance(result, set)
    assert len(result) == 2
    assert "calendar" in result
    assert "collation" in result

    if U_ICU_VERSION_MAJOR_NUM <= 64:
        # See https://unicode-org.atlassian.net/browse/ICU-20573
        pytest.skip("ICU4C<=64")

    loc = Locale("de")
    result = loc.get_keywords()
    assert len(result) == 0

    loc.set_to_bogus()
    result = loc.get_keywords()
    assert len(result) == 0


def test_get_lcid():
    # uint32_t icu::Locale::getLCID(void)
    loc = Locale("en", "US")
    assert loc.get_lcid() == 0x409


def test_get_name():
    # const char *icu::Locale::getName()
    loc = Locale("de", "DE")
    loc.set_keyword_value("collation", "phonebook")
    loc.set_keyword_value("calendar", "buddhist")
    assert loc.get_name() == "de_DE@calendar=buddhist;collation=phonebook"
    assert str(loc) == loc.get_name()


def test_get_script():
    # const char *icu::Locale::getScript()
    loc = Locale("no", "NO", "NY")
    assert len(loc.get_script()) == 0

    loc = Locale("zh", "Hans", "CN")
    assert loc.get_script() == "Hans"


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 63, reason="ICU4C<63")
def test_get_unicode_keyword_value():
    # [2]
    # template<typename StringClass>
    # StringClass icu::Locale::getUnicodeKeywordValue(
    #       StringPiece keywordName,
    #       UErrorCode &status
    # )
    loc = Locale("de@calendar=buddhist;collation=phonebook")
    value = loc.get_unicode_keyword_value("ca")
    assert isinstance(value, str)
    assert value == "buddhist"

    value = loc.get_unicode_keyword_value("co")
    assert isinstance(value, str)
    assert value == "phonebk"

    with pytest.raises(ICUError) as exc_info:
        _ = loc.get_unicode_keyword_value("cu")
    assert exc_info.value.args[0] == UErrorCode.U_ILLEGAL_ARGUMENT_ERROR

    loc.set_to_bogus()
    with pytest.raises(ICUError) as exc_info:
        _ = loc.get_unicode_keyword_value("ca")
    assert exc_info.value.args[0] == UErrorCode.U_ILLEGAL_ARGUMENT_ERROR


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 63, reason="ICU4C<63")
def test_get_unicode_keywords():
    # void icu::Locale::getUnicodeKeywords(
    #       OutputIterator iterator,
    #       UErrorCode &status
    # )
    loc = Locale("de@calendar=buddhist;collation=phonebook")
    result = loc.get_unicode_keywords()
    assert isinstance(result, set)
    assert len(result) == 2
    assert "ca" in result
    assert "co" in result

    if U_ICU_VERSION_MAJOR_NUM <= 64:
        # See https://unicode-org.atlassian.net/browse/ICU-20573
        pytest.skip("ICU4C<=64")

    loc = Locale("de")
    result = loc.get_unicode_keywords()
    assert len(result) == 0

    loc.set_to_bogus()
    result = loc.get_unicode_keywords()
    assert len(result) == 0


def test_get_variant():
    # const char *icu::Locale::getVariant()
    loc = Locale("no", "NO", "NY")
    assert loc.get_variant() == "NY"

    loc = Locale("zh", "Hans", "CN")
    assert len(loc.get_variant()) == 0


def test_hash_code():
    # int32_t icu::Locale::hashCode(void)
    test1 = Locale("en", "US")
    test2 = Locale("en", "US")

    assert test1.hash_code() == test2.hash_code()

    test1.set_keyword_value("currency", "USD")
    assert test1.hash_code() != test2.hash_code()

    test2.set_keyword_value("currency", "USD")
    assert test1.hash_code() == test2.hash_code()


def test_is_bogus():
    # UBool icu::Locale::isBogus(void)
    loc = Locale("en", "US")
    assert not loc.is_bogus()

    # void icu::Locale::setToBogus()
    loc.set_to_bogus()
    assert loc.is_bogus()


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 54, reason="ICU4C<54")
def test_is_right_to_left():
    # UBool icu::Locale::isRightToLeft()
    loc = Locale("ar")
    assert loc.is_right_to_left()

    loc = Locale("zh")
    assert not loc.is_right_to_left()


def test_locale():
    # [1]
    # icu::Locale::Locale()
    test1 = Locale()
    assert test1 == Locale.get_default()

    # [2]
    # icu::Locale::Locale(
    #       const char *language,
    #       const char *country=0,
    #       const char *variant=0,
    #       const char *keywordsAndValues=0
    # )
    test20 = Locale(None)
    assert test20 == test1

    test21 = Locale("")
    assert test21 != test1

    test22 = Locale("en")
    assert test22.get_name() == "en"

    test23 = Locale("en", "US")
    assert test23.get_name() == "en_US"

    test24 = Locale("no", "NO", "NY")
    assert test24.get_name() == "no_NO_NY"

    test25 = Locale("no", variant="NY", country="NO")
    assert test25.get_name() == test24.get_name()

    test26 = Locale("no", "NO", "NY", "currency=NOK")
    assert test26.get_name() == "no_NO_NY@currency=NOK"

    # [3]
    # icu::Locale::Locale(const Locale &other)
    test3 = Locale(test26)
    assert test3.get_name() == test26.get_name()


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 63, reason="ICU4C<63")
def test_minimize_subtags():
    # void icu::Locale::minimizeSubtags(UErrorCode &status)
    loc = Locale("zh_Hant_TW")
    loc.minimize_subtags()
    assert loc.get_name() == "zh_TW"


def test_operator():
    test1 = Locale("en", "US")
    test2 = Locale("en", "US")
    test3 = Locale("en", "GB")

    # UBool icu::Locale::operator!=(const Locale &other)
    assert not (test1 != test2)
    assert test1 != test3

    # UBool icu::Locale::operator==(const Locale &other)
    assert test1 == test2
    assert not (test1 == test3)


def test_set_default():
    default_locale = None
    try:
        default_locale = Locale.get_default()

        new_locale = Locale.get_japan()
        if new_locale == default_locale:
            new_locale = Locale.get_us()

        # static void icu::Locale::setDefault(
        #       const Locale &newLocale,
        #       UErrorCode &success
        # )
        Locale.set_default(new_locale)
        assert Locale.get_default() == new_locale

        Locale.set_default(None)
        assert Locale.get_default() == new_locale
    finally:
        if default_locale:
            Locale.set_default(default_locale)


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 49, reason="ICU4C<49")
def test_set_keyword_value():
    # [1]
    # void icu::Locale::setKeywordValue(
    #       const char *keywordName,
    #       const char *keywordValue,
    #       UErrorCode &status
    # )
    loc = Locale("de", "DE")
    loc.set_keyword_value("collation", "phonebook")
    loc.set_keyword_value("calendar", "buddhist")
    assert loc.get_name() == "de_DE@calendar=buddhist;collation=phonebook"

    with pytest.raises(ICUError) as exc_info:
        loc.set_keyword_value(None, None)
    assert exc_info.value.args[0] == UErrorCode.U_ILLEGAL_ARGUMENT_ERROR

    loc.set_keyword_value("calendar", None)
    assert loc.get_name() == "de_DE@collation=phonebook"

    loc.set_keyword_value("collation", "")
    assert loc.get_name() == "de_DE"


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 63, reason="ICU4C<63")
def test_set_unicode_keyword_value():
    # void icu::Locale::setUnicodeKeywordValue(
    #       StringPiece keywordName,
    #       StringPiece keywordValue,
    #       UErrorCode &status
    # )
    loc = Locale("de", "DE")
    loc.set_unicode_keyword_value("co", "phonebk")
    loc.set_unicode_keyword_value("ca", "buddhist")
    assert loc.get_name() == "de_DE@calendar=buddhist;collation=phonebook"

    with pytest.raises(ICUError) as exc_info:
        loc.set_unicode_keyword_value(None, None)
    assert exc_info.value.args[0] == UErrorCode.U_ILLEGAL_ARGUMENT_ERROR

    loc.set_unicode_keyword_value("ca", None)
    assert loc.get_name() == "de_DE@collation=phonebook"

    loc.set_unicode_keyword_value("co", "")
    assert loc.get_name() == "de_DE"


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 63, reason="ICU4C<63")
def test_to_language_tag():
    # [2]
    # template<typename StringClass>
    # StringClass icu::Locale::toLanguageTag(UErrorCode &status)
    loc = Locale("af@calendar=coptic;t=ar-i0-handwrit;x=foo")
    assert loc.to_language_tag() == "af-t-ar-i0-handwrit-u-ca-coptic-x-foo"



import sys
from pathlib import Path

from setuptools import setup
from setuptools.config import read_configuration

sys.path.append("")
from _build import build  # noqa

here = Path(__file__).parent.resolve()
conf_dict = read_configuration(here / "setup.cfg")
setup_kwargs = conf_dict["metadata"]
setup_kwargs.update(conf_dict.get("options", {}))
if "python_requires" in setup_kwargs:
    # Convert SpecifierSet to string
    setup_kwargs["python_requires"] = str(setup_kwargs["python_requires"])

build(setup_kwargs)
setup(**setup_kwargs)



import pytest
from icupy import (
    ErrorCode, ICUError, UErrorCode, U_ICU_VERSION_MAJOR_NUM,
    u_failure, u_get_version, u_success, u_version_from_string,
    u_version_to_string,
)


def test_icu_error():
    assert issubclass(ICUError, Exception)

    with pytest.raises(ICUError) as exc_info:
        _ = u_version_to_string([1, 2, 3])
    ex = exc_info.value
    assert isinstance(ex, ICUError)
    assert len(ex.args) == 1
    error_code = ex.args[0]
    assert isinstance(error_code, ErrorCode)

    # const char *icu::ErrorCode::errorName()
    error_name = error_code.error_name
    assert isinstance(error_name, str)
    assert error_name == "U_ILLEGAL_ARGUMENT_ERROR"

    # UErrorCode icu::ErrorCode::get()
    status = error_code.get()
    assert isinstance(status, UErrorCode)
    assert status == UErrorCode.U_ILLEGAL_ARGUMENT_ERROR

    # ErrorCode.__eq__(self, other: UErrorCode) -> bool
    assert error_code == UErrorCode.U_ILLEGAL_ARGUMENT_ERROR

    if U_ICU_VERSION_MAJOR_NUM >= 63:
        from icupy import Locale

        with pytest.raises(ICUError) as exc_info:
            _ = Locale.for_language_tag("x")
        ex = exc_info.value
        assert isinstance(ex, ICUError)
        assert len(ex.args) == 2
        assert isinstance(ex.args[0], ErrorCode)  # icu::ErrorCode
        assert isinstance(ex.args[1], str)  # An error message


# TODO: Remove test_icu_exception().
def test_icu_exception():
    """Backward compatibility test.
    ICUException is deprecated. Use ICUError instead.
    """

    from icupy import ICUException

    assert issubclass(ICUException, Exception)

    with pytest.raises(ICUException) as exc_info:
        _ = u_version_to_string([1, 2, 3])
    ex = exc_info.value
    assert isinstance(ex, ICUException)
    assert len(ex.args) == 1
    error_code = ex.args[0]
    assert isinstance(error_code, ErrorCode)

    # const char *icu::ErrorCode::errorName()
    error_name = error_code.error_name
    assert isinstance(error_name, str)
    assert error_name == "U_ILLEGAL_ARGUMENT_ERROR"

    # UErrorCode icu::ErrorCode::get()
    status = error_code.get()
    assert isinstance(status, UErrorCode)
    assert status == UErrorCode.U_ILLEGAL_ARGUMENT_ERROR

    # ErrorCode.__eq__(self, other: UErrorCode) -> bool
    assert error_code == UErrorCode.U_ILLEGAL_ARGUMENT_ERROR

    if U_ICU_VERSION_MAJOR_NUM >= 63:
        from icupy import Locale

        with pytest.raises(ICUException) as exc_info:
            _ = Locale.for_language_tag("x")
        ex = exc_info.value
        assert isinstance(ex, ICUException)
        assert len(ex.args) == 2
        assert isinstance(ex.args[0], ErrorCode)  # icu::ErrorCode
        assert isinstance(ex.args[1], str)  # Error message


def test_u_failure():
    assert not u_failure(UErrorCode.U_USING_FALLBACK_WARNING)
    assert not u_failure(UErrorCode.U_ZERO_ERROR)
    assert u_failure(UErrorCode.U_ILLEGAL_ARGUMENT_ERROR)


def test_u_get_version():
    # void u_getVersion(UVersionInfo versionArray)
    version_array = u_get_version()
    assert isinstance(version_array, list)
    assert len(version_array) == 4
    assert version_array[0] > 0
    assert version_array[1] >= 0
    assert version_array[2] >= 0
    assert version_array[3] >= 0


def test_u_success():
    assert u_success(UErrorCode.U_USING_FALLBACK_WARNING)
    assert u_success(UErrorCode.U_ZERO_ERROR)
    assert not u_success(UErrorCode.U_ILLEGAL_ARGUMENT_ERROR)


def test_u_version_from_string():
    # void u_versionFromString(
    #       UVersionInfo versionArray,
    #       const char *versionString
    # )
    version_array = u_version_from_string("123.45.67.89")
    assert isinstance(version_array, list)
    assert len(version_array) == 4
    assert version_array[0] == 123
    assert version_array[1] == 45
    assert version_array[2] == 67
    assert version_array[3] == 89


def test_u_version_to_string():
    # void u_versionToString(
    #       const UVersionInfo versionArray,
    #       char *versionString
    # )
    version_string = u_version_to_string([123, 45, 67, 89])
    assert isinstance(version_string, str)
    assert version_string == "123.45.67.89"

    with pytest.raises(ICUError) as exc_info:
        _ = u_version_to_string([1, 2, 3])
    assert exc_info.value.args[0] == UErrorCode.U_ILLEGAL_ARGUMENT_ERROR

    with pytest.raises(ICUError) as exc_info:
        _ = u_version_to_string([1, 2, 3, 4, 5])
    assert exc_info.value.args[0] == UErrorCode.U_ILLEGAL_ARGUMENT_ERROR

    with pytest.raises(TypeError):
        _ = u_version_to_string([256, 0, 0, 0])  # [0, 255]



import pytest
from icupy import (
    ICUError, StringCharacterIterator, UErrorCode, UTextVector,
    U_SENTINEL, UnicodeString,
    utext_char32_at, utext_clone, utext_close, utext_copy, utext_current32,
    utext_equals, utext_extract, utext_freeze, utext_get_native_index,
    utext_get_previous_native_index, utext_has_meta_data,
    utext_is_length_expensive, utext_is_writable, utext_move_index32,
    utext_native_length, utext_next32, utext_next32_from,
    utext_open_character_iterator, utext_open_const_unicode_string,
    utext_open_replaceable, utext_open_uchars, utext_open_unicode_string,
    utext_open_utf8, utext_previous32, utext_previous32_from, utext_replace,
    utext_set_native_index,
)

from . import gc


def test_api():
    s = "\x41\U0001f338\x42"
    with gc(utext_open_uchars(None, s, -1), utext_close) as ut:
        assert ut.magic
        assert ut.flags
        assert ut.provider_properties
        assert ut.p_funcs
        assert utext_char32_at(ut, 0) == 0x41
        assert utext_char32_at(ut, 1) == 0x1f338
        assert utext_char32_at(ut, 2) == 0x1f338
        assert utext_char32_at(ut, 3) == 0x42
        assert utext_char32_at(ut, 4) == U_SENTINEL

        # int32_t utext_extract(UText *ut,
        #                       int64_t nativeStart,
        #                       int64_t nativeLimit,
        #                       UChar *dest,
        #                       int32_t destCapacity,
        #                       UErrorCode *status
        # )
        dest = utext_extract(ut, 1, 3)
        assert isinstance(dest, str)
        assert dest == "\U0001f338"

        utext_set_native_index(ut, 0)
        assert utext_current32(ut) == 0x41
        assert utext_get_native_index(ut) == 0
        assert utext_get_previous_native_index(ut) == 0

        assert not utext_has_meta_data(ut)
        assert not utext_is_length_expensive(ut)
        assert not utext_is_writable(ut)

        utext_move_index32(ut, 1)
        assert utext_native_length(ut) == 4
        assert utext_next32(ut) == 0x1f338
        assert utext_next32(ut) == 0x42
        assert utext_next32(ut) == U_SENTINEL

        assert utext_next32_from(ut, 2) == 0x1f338
        assert utext_next32(ut) == 0x42

        assert utext_previous32(ut) == 0x42
        assert utext_previous32(ut) == 0x1f338
        assert utext_previous32(ut) == 0x41
        assert utext_previous32(ut) == U_SENTINEL

        assert utext_previous32_from(ut, 3) == 0x1f338
        assert utext_previous32(ut) == 0x41


def test_clone_uchars():
    s1 = "ABC"
    s2 = "abc"
    ut1 = utext_open_uchars(None, s1, -1)
    assert not utext_is_writable(ut1)

    # Deep clone
    ut2 = utext_clone(None, ut1, True, True)
    assert ut2 is not None
    assert ut2 != ut1
    assert not utext_is_writable(ut2)
    assert utext_extract(ut2, 0, utext_native_length(ut2)) == "ABC"
    assert utext_close(ut2) is None

    ut2 = utext_open_uchars(None, s2, -1)
    ut3 = utext_clone(ut2, ut1, True, True)
    assert ut3 is not None
    assert ut3 == ut2
    assert ut3 != ut1
    assert not utext_is_writable(ut3)
    assert utext_extract(ut3, 0, utext_native_length(ut3)) == "ABC"
    assert utext_close(ut2) is None
    assert utext_close(ut3) == ut2

    ut2 = utext_open_uchars(None, s2, -1)
    utext_clone(ut2, ut1, True, False)
    assert ut2 != ut1
    assert not utext_is_writable(ut2)
    assert utext_extract(ut2, 0, utext_native_length(ut2)) == "ABC"
    assert utext_close(ut2) is None

    # Shallow clone
    ut4 = utext_clone(None, ut1, False, True)
    assert ut4 is not None
    assert ut4 != ut1
    assert not utext_is_writable(ut4)
    assert utext_extract(ut4, 0, utext_native_length(ut4)) == "ABC"
    assert utext_close(ut4) is None

    ut5 = utext_open_uchars(None, s2, -1)
    ut6 = utext_clone(ut5, ut1, False, True)
    assert ut6 is not None
    assert ut6 == ut5
    assert ut6 != ut1
    assert not utext_is_writable(ut6)
    assert utext_extract(ut6, 0, utext_native_length(ut6)) == "ABC"
    assert utext_close(ut5) is None
    assert utext_close(ut6) == ut5

    ut5 = utext_open_uchars(None, s2, -1)
    utext_clone(ut5, ut1, False, False)
    assert ut5 != ut1
    assert not utext_is_writable(ut5)
    assert utext_extract(ut5, 0, utext_native_length(ut5)) == "ABC"
    assert utext_close(ut5) is None

    assert utext_close(ut1) is None


def test_clone_unicode_string():
    s1 = UnicodeString("ABC")
    s2 = UnicodeString("abc")
    ut1 = utext_open_unicode_string(None, s1)
    assert utext_is_writable(ut1)

    # Deep clone
    ut2 = utext_clone(None, ut1, True, True)
    assert ut2 is not None
    assert ut2 != ut1
    assert not utext_is_writable(ut2)
    assert utext_extract(ut2, 0, utext_native_length(ut2)) == "ABC"
    assert utext_close(ut2) is None

    ut2 = utext_open_unicode_string(None, s2)
    ut3 = utext_clone(ut2, ut1, True, True)
    assert ut3 is not None
    assert ut3 == ut2
    assert ut3 != ut1
    assert not utext_is_writable(ut3)
    assert utext_extract(ut3, 0, utext_native_length(ut3)) == "ABC"
    assert utext_close(ut2) is None
    assert utext_close(ut3) == ut2

    ut2 = utext_open_unicode_string(None, s2)
    utext_clone(ut2, ut1, True, False)
    assert ut2 != ut1
    assert utext_is_writable(ut2)
    assert utext_extract(ut2, 0, utext_native_length(ut2)) == "ABC"
    assert utext_close(ut2) is None

    # Shallow clone
    ut4 = utext_clone(None, ut1, False, True)
    assert ut4 is not None
    assert ut4 != ut1
    assert not utext_is_writable(ut4)
    assert utext_extract(ut4, 0, utext_native_length(ut4)) == "ABC"
    assert utext_close(ut4) is None

    ut5 = utext_open_unicode_string(None, s2)
    ut6 = utext_clone(ut5, ut1, False, True)
    assert ut6 is not None
    assert ut6 == ut5
    assert ut6 != ut1
    assert not utext_is_writable(ut6)
    assert utext_extract(ut6, 0, utext_native_length(ut6)) == "ABC"
    assert utext_close(ut5) is None
    assert utext_close(ut6) == ut5

    ut5 = utext_open_unicode_string(None, s2)
    utext_clone(ut5, ut1, False, False)
    assert ut5 != ut1
    assert utext_is_writable(ut5)
    assert utext_extract(ut5, 0, utext_native_length(ut5)) == "ABC"
    assert utext_close(ut5) is None

    assert utext_close(ut1) is None


def test_copy():
    s1 = "ABC"
    with gc(utext_open_uchars(None, s1, -1), utext_close) as ut1:
        assert not utext_is_writable(ut1)

        with pytest.raises(ICUError) as exc_info:
            utext_copy(ut1, 0, 2, 2, False)
        assert exc_info.value.args[0] == UErrorCode.U_NO_WRITE_PERMISSION

    s2 = UnicodeString(s1)
    with gc(utext_open_unicode_string(None, s2), utext_close) as ut2:
        assert utext_is_writable(ut2)

        # Copy: |AB|C -> AB|AB|C
        utext_copy(ut2, 0, 2, 2, False)
        assert utext_extract(ut2, 0, utext_native_length(ut2)) == "ABABC"

        # Move: A|BA|BC -> ABC|BA|
        utext_copy(ut2, 1, 3, 5, True)
        assert utext_extract(ut2, 0, utext_native_length(ut2)) == "ABCBA"


def test_equals():
    s1 = "ABC"
    with gc(utext_open_uchars(None, s1, -1), utext_close) as ut1:
        with gc(utext_clone(None, ut1, False, True), utext_close) as ut2:
            utext_set_native_index(ut1, 0)
            utext_set_native_index(ut2, 0)
            assert utext_equals(ut1, ut2)

            assert utext_next32(ut1) != U_SENTINEL
            assert not utext_equals(ut1, ut2)

            assert utext_next32(ut2) != U_SENTINEL
            assert utext_equals(ut1, ut2)


def test_freeze():
    s = UnicodeString("ABC")
    with gc(utext_open_unicode_string(None, s), utext_close) as ut:
        assert utext_is_writable(ut)
        utext_freeze(ut)
        assert not utext_is_writable(ut)


def test_open_character_iterator():
    src1 = UnicodeString("\x41\U0001f338\x42")
    src2 = UnicodeString("\x61\U0001f338\x62")
    it1 = StringCharacterIterator(src1)
    it2 = StringCharacterIterator(src2)
    with gc(utext_open_character_iterator(None, it1), utext_close) as ut1:
        assert not utext_is_writable(ut1)

        dest1 = utext_extract(ut1, 0, utext_native_length(ut1))
        assert dest1 == "A\U0001f338B"

        # Reset
        ut2 = utext_open_character_iterator(ut1, it2)
        assert ut2 == ut1

        dest1 = utext_extract(ut1, 0, utext_native_length(ut1))
        assert dest1 == "a\U0001f338b"

        dest2 = utext_extract(ut2, 0, utext_native_length(ut2))
        assert dest2 == "a\U0001f338b"

        # Keep-alive test
        utext_open_character_iterator(ut1, it1)
        dest1 = utext_extract(ut1, 0, utext_native_length(ut1))
        assert dest1 == "A\U0001f338B"


def test_open_const_unicode_string():
    s1 = UnicodeString("\x41\U0001f338\x42")
    s2 = UnicodeString("\x61\U0001f338\x62")
    with gc(utext_open_const_unicode_string(None, s1), utext_close) as ut1:
        assert not utext_is_writable(ut1)

        dest1 = utext_extract(ut1, 0, utext_native_length(ut1))
        assert dest1 == "A\U0001f338B"

        # Reset
        ut2 = utext_open_const_unicode_string(ut1, s2)
        assert ut2 == ut1

        dest1 = utext_extract(ut1, 0, utext_native_length(ut1))
        assert dest1 == "a\U0001f338b"

        dest2 = utext_extract(ut2, 0, utext_native_length(ut2))
        assert dest2 == "a\U0001f338b"

        # Keep-alive test
        utext_open_const_unicode_string(ut1, s1)
        dest1 = utext_extract(ut1, 0, utext_native_length(ut1))
        assert dest1 == "A\U0001f338B"


def test_open_replaceable():
    s1 = UnicodeString("\x41\U0001f338\x42")
    s2 = UnicodeString("\x61\U0001f338\x62")
    with gc(utext_open_replaceable(None, s1), utext_close) as ut1:
        assert utext_is_writable(ut1)

        dest1 = utext_extract(ut1, 0, utext_native_length(ut1))
        assert dest1 == "A\U0001f338B"

        # Reset
        ut2 = utext_open_replaceable(ut1, s2)
        assert ut2 == ut1

        dest1 = utext_extract(ut1, 0, utext_native_length(ut1))
        assert dest1 == "a\U0001f338b"

        dest2 = utext_extract(ut2, 0, utext_native_length(ut2))
        assert dest2 == "a\U0001f338b"

        # Keep-alive test
        utext_open_replaceable(ut1, s1)
        dest1 = utext_extract(ut1, 0, utext_native_length(ut1))
        assert dest1 == "A\U0001f338B"


def test_open_uchars():
    s1 = "\x41\U0001f338\x42"
    s2 = "\x61\U0001f338\x62"
    with gc(utext_open_uchars(None, s1, -1), utext_close) as ut1:
        assert not utext_is_writable(ut1)

        dest1 = utext_extract(ut1, 0, utext_native_length(ut1))
        assert dest1 == "A\U0001f338B"

        # Reset
        ut2 = utext_open_uchars(ut1, s2, -1)
        assert ut2 == ut1

        dest1 = utext_extract(ut1, 0, utext_native_length(ut1))
        assert dest1 == "a\U0001f338b"

        dest2 = utext_extract(ut2, 0, utext_native_length(ut2))
        assert dest2 == "a\U0001f338b"

        # Keep-alive test
        utext_open_uchars(ut1, s1, -1)
        dest1 = utext_extract(ut1, 0, utext_native_length(ut1))
        assert dest1 == "A\U0001f338B"

        # NULL string
        utext_open_uchars(ut1, None, -1)
        dest1 = utext_extract(ut1, 0, 4)
        assert len(dest1) == 0


def test_open_unicode_string():
    s1 = UnicodeString("\x41\U0001f338\x42")
    s2 = UnicodeString("\x61\U0001f338\x62")
    with gc(utext_open_unicode_string(None, s1), utext_close) as ut1:
        assert utext_is_writable(ut1)

        dest1 = utext_extract(ut1, 0, utext_native_length(ut1))
        assert dest1 == "A\U0001f338B"

        # Reset
        ut2 = utext_open_unicode_string(ut1, s2)
        assert ut2 == ut1

        dest1 = utext_extract(ut1, 0, utext_native_length(ut1))
        assert dest1 == "a\U0001f338b"

        dest2 = utext_extract(ut2, 0, utext_native_length(ut2))
        assert dest2 == "a\U0001f338b"

        # Keep-alive test
        utext_open_unicode_string(ut1, s1)
        dest1 = utext_extract(ut1, 0, utext_native_length(ut1))
        assert dest1 == "A\U0001f338B"


def test_open_utf8():
    s1 = "\x41\U0001f338\x42"
    s2 = "\x61\U0001f338\x62"
    with gc(utext_open_utf8(None, s1, -1), utext_close) as ut1:
        assert not utext_is_writable(ut1)

        dest1 = utext_extract(ut1, 0, utext_native_length(ut1))
        assert dest1 == "A\U0001f338B"

        # Reset
        ut2 = utext_open_utf8(ut1, s2, -1)
        assert ut2 == ut1

        dest1 = utext_extract(ut1, 0, utext_native_length(ut1))
        assert dest1 == "a\U0001f338b"

        dest2 = utext_extract(ut2, 0, utext_native_length(ut2))
        assert dest2 == "a\U0001f338b"

        # Keep-alive test
        utext_open_utf8(ut1, s1, -1)
        dest1 = utext_extract(ut1, 0, utext_native_length(ut1))
        assert dest1 == "A\U0001f338B"

        # NULL string
        utext_open_utf8(ut1, None, -1)
        dest1 = utext_extract(ut1, 0, 4)
        assert len(dest1) == 0


def test_replace():
    s1 = "ABC"
    with gc(utext_open_uchars(None, s1, -1), utext_close) as ut1:
        assert not utext_is_writable(ut1)

        with pytest.raises(ICUError) as exc_info:
            utext_replace(ut1, 1, 2, "abc", -1)
        assert exc_info.value.args[0] == UErrorCode.U_NO_WRITE_PERMISSION

    s2 = UnicodeString(s1)
    with gc(utext_open_unicode_string(None, s2), utext_close) as ut2:
        assert utext_is_writable(ut2)

        assert utext_replace(ut2, 1, 2, "abc", -1) == 2
        assert utext_extract(ut2, 0, utext_native_length(ut2)) == "AabcC"


def test_utext_vector():
    t = UTextVector(3)
    assert len(t) == 3

    dest1 = utext_extract(t[0], 0, utext_native_length(t[0]))
    dest2 = utext_extract(t[1], 0, utext_native_length(t[1]))
    dest3 = utext_extract(t[2], 0, utext_native_length(t[2]))
    assert len(dest1) == 0
    assert len(dest2) == 0
    assert len(dest3) == 0

    assert utext_is_writable(t[0])
    assert utext_is_writable(t[1])
    assert utext_is_writable(t[2])

    utext_replace(t[0], 0, utext_native_length(t[0]), "foo", -1)
    utext_replace(t[1], 0, utext_native_length(t[1]), "bar", -1)
    utext_replace(t[2], 0, utext_native_length(t[2]), "baz", -1)

    assert utext_extract(t[0], 0, utext_native_length(t[0])) == "foo"
    assert utext_extract(t[1], 0, utext_native_length(t[1])) == "bar"
    assert utext_extract(t[2], 0, utext_native_length(t[2])) == "baz"

    # ut0, ut1, ut2 = t[0], t[1], t[2]
    # del t
    # assert ut0.magic == 0
    # assert ut1.magic == 0
    # assert ut2.magic == 0

    s = [UnicodeString("foo"), UnicodeString("bar"), UnicodeString("baz")]
    t = UTextVector(s)
    assert len(t) == 3

    assert utext_extract(t[0], 0, utext_native_length(t[0])) == s[0]
    assert utext_extract(t[1], 0, utext_native_length(t[1])) == s[1]
    assert utext_extract(t[2], 0, utext_native_length(t[2])) == s[2]

    assert utext_is_writable(t[0])
    assert utext_is_writable(t[1])
    assert utext_is_writable(t[2])
